@@ -20,7 +20,6 @@
 
 #include "../simple_packet_parser_ng.hpp"
 
-// For support: IPPROTO_TCP, IPPROTO_ICMP, IPPROTO_UDP
 #include <netinet/in.h>
 #include <sys/socket.h>
 #include <sys/types.h>
@@ -39,7 +38,9 @@
 
 #include <linux/filter.h> // for struct sock_filter
 
-bool afpacket_read_packet_length_from_ip_header = false;
+#include "../fastnetmon_configuration_scheme.hpp"
+
+extern fastnetmon_configuration_t fastnetmon_global_configuration;
 
 // Get log4cpp logger from main programme
 extern log4cpp::Category& logger;
@@ -50,7 +51,7 @@ extern uint64_t total_unparsed_packets;
 // Global configuration map
 extern std::map<std::string, std::string> configuration_map;
 
-// This variable name should be uniq for every plugin!
+// This variable name should be unique for every plugin!
 process_packet_pointer afpacket_process_func_ptr = NULL;
 
 std::string socket_received_packets_desc = "Number of received packets";
@@ -76,8 +77,11 @@ uint32_t mirror_af_packet_custom_sampling_rate = 1;
 
 // 4194304 bytes
 unsigned int blocksiz = 1 << 22;
+
 // 2048 bytes
 unsigned int framesiz = 1 << 11;
+
+// Number of blocks
 unsigned int blocknum = 64;
 
 struct block_desc {
@@ -102,6 +106,11 @@ int fanout_type = PACKET_FANOUT_CPU;
 #define PACKET_FANOUT_QM 5
 #endif
 
+void start_af_packet_capture_for_interface(std::string capture_interface, int fanout_group_id, unsigned int num_cpus);
+
+// We keep all active AF_PACKET sockets here
+std::vector<int> active_af_packet_sockets;
+std::mutex active_af_packet_sockets_mutex;
 
 int get_fanout_by_name(std::string fanout_name) {
     if (fanout_name == "" || fanout_name == "cpu") {
@@ -124,205 +133,32 @@ int get_fanout_by_name(std::string fanout_name) {
     }
 }
 
-void flush_block(struct block_desc* pbd) {
-    pbd->h1.block_status = TP_STATUS_KERNEL;
-}
-
-void walk_block(struct block_desc* pbd, const int block_num) {
-    int num_pkts        = pbd->h1.num_pkts, i;
-    unsigned long bytes = 0;
-    struct tpacket3_hdr* ppd;
-
-    ppd = (struct tpacket3_hdr*)((uint8_t*)pbd + pbd->h1.offset_to_first_pkt);
-    for (i = 0; i < num_pkts; ++i) {
-        bytes += ppd->tp_snaplen;
-
-        // struct ethhdr *eth = (struct ethhdr *) ((uint8_t *) ppd + ppd->tp_mac);
-        // Print packets
-
-        u_char* data_pointer = (u_char*)((uint8_t*)ppd + ppd->tp_mac);
-
-        simple_packet_t packet;
-
-        packet.source       = MIRROR;
-        packet.arrival_time = current_inaccurate_time;
-        packet.sample_ratio = 1; //-V1048
-
-
-        // Override default sample rate by rate specified in configuration
-        if (mirror_af_packet_custom_sampling_rate > 1) {
-            packet.sample_ratio = mirror_af_packet_custom_sampling_rate;
-        }
-
-        // Not enabled by default
-        bool af_packet_extract_tunnel_traffic = false;
-
-        auto result = parse_raw_packet_to_simple_packet_full_ng((u_char*)data_pointer, ppd->tp_snaplen, ppd->tp_snaplen,
-                                                                packet, af_packet_extract_tunnel_traffic,
-                                                                afpacket_read_packet_length_from_ip_header);
-
-        if (result != network_data_stuctures::parser_code_t::success) {
-            // This counter resets for speed calculation every second
-            total_unparsed_packets++;
-            af_packet_packets_unparsed++;
-
-            logger << log4cpp::Priority::DEBUG << "Cannot parse packet using ng parser: " << parser_code_to_string(result);
-        } else {
-            af_packet_packets_parsed++;
-            afpacket_process_func_ptr(packet);
-        }
-
-        // Move pointer to next packet
-        ppd = (struct tpacket3_hdr*)((uint8_t*)ppd + ppd->tp_next_offset);
-    }
-}
-
-bool setup_socket(std::string interface_name, bool enable_fanout, int fanout_group_id) {
-    // More details here: http://man7.org/linux/man-pages/man7/packet.7.html
-    // We could use SOCK_RAW or SOCK_DGRAM for second argument
-    // SOCK_RAW - raw packets pass from the kernel
-    // SOCK_DGRAM - some amount of processing
-    // Third argument manage ether type of captured packets
-    int packet_socket = socket(AF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
-
-    if (packet_socket == -1) {
-        logger << log4cpp::Priority::ERROR << "Can't create AF_PACKET socket. Error number: " << errno
-               << " error text: " << strerror(errno);
-        return false;
-    }
-
-    // We whould use V3 bcause it could read/pool in per block basis instead per packet
-    int version                   = TPACKET_V3;
-    int setsockopt_packet_version = setsockopt(packet_socket, SOL_PACKET, PACKET_VERSION, &version, sizeof(version));
-
-    if (setsockopt_packet_version < 0) {
-        logger << log4cpp::Priority::ERROR << "Can't set packet v3 version";
-        return false;
-    }
-
-    int interface_number = 0;
-
-    bool get_interface_number_result = get_interface_number_by_device_name(packet_socket, interface_name, interface_number);
-
-    if (!get_interface_number_result) {
-        logger << log4cpp::Priority::ERROR << "Can't get interface number by interface name for " << interface_name;
-        return false;
-    }
-
-    // Switch to PROMISC mode
-    struct packet_mreq sock_params;
-    memset(&sock_params, 0, sizeof(sock_params));
-    sock_params.mr_type    = PACKET_MR_PROMISC;
-    sock_params.mr_ifindex = interface_number;
-
-    int set_promisc = setsockopt(packet_socket, SOL_PACKET, PACKET_ADD_MEMBERSHIP, (void*)&sock_params, sizeof(sock_params));
-
-    if (set_promisc == -1) {
-        logger << log4cpp::Priority::ERROR << "Can't enable promisc mode";
-        return false;
-    }
-
-    struct sockaddr_ll bind_address;
-    memset(&bind_address, 0, sizeof(bind_address));
-
-    bind_address.sll_family   = AF_PACKET;
-    bind_address.sll_protocol = htons(ETH_P_ALL);
-    bind_address.sll_ifindex  = interface_number;
-
-    // We will follow http://yusufonlinux.blogspot.ru/2010/11/data-link-access-and-zero-copy.html
-    // And this: https://www.kernel.org/doc/Documentation/networking/packet_mmap.txt
-
-    struct tpacket_req3 req;
-    memset(&req, 0, sizeof(req));
-
-    req.tp_block_size = blocksiz;
-    req.tp_frame_size = framesiz;
-    req.tp_block_nr   = blocknum;
-    req.tp_frame_nr   = (blocksiz * blocknum) / framesiz;
-
-    req.tp_retire_blk_tov   = 60; // Timeout in msec
-    req.tp_feature_req_word = TP_FT_REQ_FILL_RXHASH;
-
-    int setsockopt_rx_ring = setsockopt(packet_socket, SOL_PACKET, PACKET_RX_RING, (void*)&req, sizeof(req));
-
-    if (setsockopt_rx_ring == -1) {
-        logger << log4cpp::Priority::ERROR << "Can't enable RX_RING for AF_PACKET socket";
-        return false;
-    }
-
-    // We use per thread structures
-    uint8_t* mapped_buffer = NULL;
-    struct iovec* rd       = NULL;
-
-    mapped_buffer = (uint8_t*)mmap(NULL, req.tp_block_size * req.tp_block_nr, PROT_READ | PROT_WRITE,
-                                   MAP_SHARED | MAP_LOCKED, packet_socket, 0);
-
-    if (mapped_buffer == MAP_FAILED) {
-        logger << log4cpp::Priority::ERROR << "MMAP failed errno: " << errno << " error: " << strerror(errno);
-        return false;
-    }
-
-    // Allocate iov structure for each block
-    rd = (struct iovec*)malloc(req.tp_block_nr * sizeof(struct iovec));
-
-    // Initilise iov structures
-    for (unsigned int i = 0; i < req.tp_block_nr; ++i) {
-        rd[i].iov_base = mapped_buffer + (i * req.tp_block_size);
-        rd[i].iov_len  = req.tp_block_size;
-    }
-
-    int bind_result = bind(packet_socket, (struct sockaddr*)&bind_address, sizeof(bind_address));
-
-    if (bind_result == -1) {
-        logger << log4cpp::Priority::ERROR << "Can't bind to AF_PACKET socket";
-        return false;
-    }
-
-    if (enable_fanout) {
-        int fanout_arg = (fanout_group_id | (fanout_type << 16));
-
-        int setsockopt_fanout = setsockopt(packet_socket, SOL_PACKET, PACKET_FANOUT, &fanout_arg, sizeof(fanout_arg));
-
-        if (setsockopt_fanout < 0) {
-            logger << log4cpp::Priority::ERROR << "Can't configure fanout error number: " << errno
-                   << " error: " << strerror(errno);
-            return false;
-        }
-    }
-
-    unsigned int current_block_num = 0;
-
-    struct pollfd pfd;
-    memset(&pfd, 0, sizeof(pfd));
-
-    pfd.fd      = packet_socket;
-    pfd.events  = POLLIN | POLLERR;
-    pfd.revents = 0;
-
-    while (true) {
-        struct block_desc* pbd = (struct block_desc*)rd[current_block_num].iov_base;
-
-        if ((pbd->h1.block_status & TP_STATUS_USER) == 0) {
-            poll(&pfd, 1, -1);
-
-            continue;
-        }
-
-        walk_block(pbd, current_block_num);
-        flush_block(pbd);
-        current_block_num = (current_block_num + 1) % blocknum;
-    }
-
-    return true;
-}
-
-void start_af_packet_capture(std::string interface_name, bool enable_fanout, int fanout_group_id) {
-    setup_socket(interface_name, enable_fanout, fanout_group_id);
-}
-
 std::vector<system_counter_t> get_af_packet_stats() {
     std::vector<system_counter_t> system_counter;
 
+    if (true) {
+        // We should iterate over all sockets because every socket has independent counter
+        std::lock_guard<std::mutex> lock_guard(active_af_packet_sockets_mutex);
+
+        for (auto socket : active_af_packet_sockets) {
+            struct tpacket_stats_v3 stats3;
+            memset(&stats3, 0, sizeof(struct tpacket_stats_v3));
+
+            socklen_t structure_length = sizeof(struct tpacket_stats_v3);
+
+            // Each per socket structure will reset to zero after reading
+            // We need to investigate that this system call is thread safe. I did not find any evidence of it and more detailed research is needed
+            int res = getsockopt(socket, SOL_PACKET, PACKET_STATISTICS, &stats3, &structure_length);
+
+            if (res != 0) {
+                logger << log4cpp::Priority::ERROR << "Cannot read socket stats with error code: " << res;
+            } else {
+                socket_received_packets += stats3.tp_packets;
+                socket_dropped_packets += stats3.tp_drops;
+            }
+        }
+    }
+
     system_counter.push_back(system_counter_t("af_packet_socket_received_packets", socket_received_packets,
                                               metric_type_t::counter, socket_received_packets_desc));
     system_counter.push_back(system_counter_t("af_packet_socket_dropped_packets", socket_dropped_packets,
@@ -341,50 +177,255 @@ std::vector<system_counter_t> get_af_packet_stats() {
 }
 
 
-// Could get some speed up on NUMA servers
-bool afpacket_execute_strict_cpu_affinity = false;
+void flush_block(struct block_desc* pbd) {
+    pbd->h1.block_status = TP_STATUS_KERNEL;
+}
+
+void walk_block(struct block_desc* pbd) {
+    struct tpacket3_hdr* ppd = (struct tpacket3_hdr*)((uint8_t*)pbd + pbd->h1.offset_to_first_pkt);
+
+    for (uint32_t i = 0; i < pbd->h1.num_pkts; ++i) {
+        af_packet_packets_raw++;
+
+        // struct ethhdr *eth = (struct ethhdr *) ((uint8_t *) ppd + ppd->tp_mac);
+        // Print packets
+
+        u_char* data_pointer = (u_char*)((uint8_t*)ppd + ppd->tp_mac);
+
+        simple_packet_t packet;
+
+        packet.source       = MIRROR;
+        packet.arrival_time = current_inaccurate_time;
+        packet.sample_ratio = 1; //-V1048
+
+
+        // Override default sample rate by rate specified in configuration
+        if (mirror_af_packet_custom_sampling_rate > 1) {
+            packet.sample_ratio = mirror_af_packet_custom_sampling_rate;
+        }
+
+        auto result =
+            parse_raw_packet_to_simple_packet_full_ng((u_char*)data_pointer, ppd->tp_snaplen, ppd->tp_snaplen, packet,
+                                                      fastnetmon_global_configuration.af_packet_extract_tunnel_traffic,
+                                                      fastnetmon_global_configuration.af_packet_read_packet_length_from_ip_header);
+
+        if (result != network_data_stuctures::parser_code_t::success) {
+            // This counter resets for speed calculation every second
+            total_unparsed_packets++;
+            af_packet_packets_unparsed++;
+
+            logger << log4cpp::Priority::DEBUG << "Cannot parse packet using ng parser: " << parser_code_to_string(result);
+        } else {
+            af_packet_packets_parsed++;
+            afpacket_process_func_ptr(packet);
+        }
+
+        // Move pointer to next packet
+        ppd = (struct tpacket3_hdr*)((uint8_t*)ppd + ppd->tp_next_offset);
+    }
+}
+
+void read_packets_from_socket(int packet_socket, struct iovec* rd);
+
+// Configures socket and starts traffic capture
+bool setup_socket(std::string interface_name, bool enable_fanout, int fanout_group_id) {
+    // More details here: http://man7.org/linux/man-pages/man7/packet.7.html
+    // We could use SOCK_RAW or SOCK_DGRAM for second argument
+    // SOCK_RAW - raw packets pass from the kernel
+    // SOCK_DGRAM - some amount of processing
+    // Third argument manage ether type of captured packets
+    int packet_socket = socket(AF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
+
+    if (packet_socket == -1) {
+        logger << log4cpp::Priority::ERROR << "Can't create AF_PACKET socket. Error number: " << errno
+               << " error text: " << strerror(errno);
+        return false;
+    }
+
+    if (true) {
+        // Add socket to global structure, we will use it to get statistics for each of them
+        std::lock_guard<std::mutex> lock_guard(active_af_packet_sockets_mutex);
+        active_af_packet_sockets.push_back(packet_socket);
+    }
+
+    // We should use V3 because it could read/pool in per block basis instead per
+    // packet
+    int version                   = TPACKET_V3;
+    int setsockopt_packet_version = setsockopt(packet_socket, SOL_PACKET, PACKET_VERSION, &version, sizeof(version));
+
+    if (setsockopt_packet_version < 0) {
+        logger << log4cpp::Priority::ERROR << "Can't set packet v3 version for " << interface_name;
+        return false;
+    }
+
+    int interface_number = 0;
+
+    bool get_interface_number_result = get_interface_number_by_device_name(packet_socket, interface_name, interface_number);
+
+    if (!get_interface_number_result) {
+        logger << log4cpp::Priority::ERROR << "Can't get interface number by interface name for " << interface_name;
+        return false;
+    }
+
+    // Without PACKET_ADD_MEMBERSHIP step AF_PACKET will capture traffic on all interfaces which is not exactly what we
+    // want in this case We need to specify exact interface we're interested in here
+
+    // Switch to PROMISC mode
+    struct packet_mreq sock_params;
+    memset(&sock_params, 0, sizeof(sock_params));
+    sock_params.mr_type    = PACKET_MR_PROMISC;
+    sock_params.mr_ifindex = interface_number;
+
+    int set_promisc = setsockopt(packet_socket, SOL_PACKET, PACKET_ADD_MEMBERSHIP, (void*)&sock_params, sizeof(sock_params));
+
+    if (set_promisc == -1) {
+        logger << log4cpp::Priority::ERROR << "Can't enable promisc mode for " << interface_name;
+        return false;
+    }
+
+    // We will follow
+    // http://yusufonlinux.blogspot.ru/2010/11/data-link-access-and-zero-copy.html
+    // And this:
+    // https://www.kernel.org/doc/Documentation/networking/packet_mmap.txt
+    struct tpacket_req3 req;
+    memset(&req, 0, sizeof(req));
+
+    req.tp_block_size = blocksiz;
+    req.tp_frame_size = framesiz;
+    req.tp_block_nr   = blocknum;
+    req.tp_frame_nr   = (blocksiz * blocknum) / framesiz;
+
+    req.tp_retire_blk_tov   = 60; // Timeout in msec
+    req.tp_feature_req_word = TP_FT_REQ_FILL_RXHASH;
+
+    int setsockopt_rx_ring = setsockopt(packet_socket, SOL_PACKET, PACKET_RX_RING, (void*)&req, sizeof(req));
+
+    if (setsockopt_rx_ring == -1) {
+        logger << log4cpp::Priority::ERROR << "Can't enable RX_RING for AF_PACKET socket for " << interface_name;
+        return false;
+    }
+
+    size_t buffer_size = req.tp_block_size * req.tp_block_nr;
+
+    logger << log4cpp::Priority::DEBUG << "Allocating " << buffer_size << " byte buffer for AF_PACKET interface: " << interface_name;
+
+    uint8_t* mapped_buffer =
+        (uint8_t*)mmap(NULL, buffer_size, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_LOCKED, packet_socket, 0);
+
+    if (mapped_buffer == MAP_FAILED) {
+        logger << log4cpp::Priority::ERROR << "mmap failed errno: " << errno << " error: " << strerror(errno);
+        return false;
+    }
+
+    // Allocate iov structure for each block
+    struct iovec* rd = (struct iovec*)malloc(req.tp_block_nr * sizeof(struct iovec));
+
+    if (rd == NULL) {
+        logger << log4cpp::Priority::ERROR << "Cannot allocate memory for iovecs for " << interface_name;
+        return false;
+    }
+
+    // Initialise iov structures
+    for (unsigned int i = 0; i < req.tp_block_nr; ++i) {
+        rd[i].iov_base = mapped_buffer + (i * req.tp_block_size);
+        rd[i].iov_len  = req.tp_block_size;
+    }
+
+    struct sockaddr_ll bind_address;
+    memset(&bind_address, 0, sizeof(bind_address));
+
+    bind_address.sll_family   = AF_PACKET;
+    bind_address.sll_protocol = htons(ETH_P_ALL);
+    bind_address.sll_ifindex  = interface_number;
+
+    int bind_result = bind(packet_socket, (struct sockaddr*)&bind_address, sizeof(bind_address));
+
+    if (bind_result == -1) {
+        logger << log4cpp::Priority::ERROR << "Can't bind to AF_PACKET socket for " << interface_name;
+        return false;
+    }
+
+    if (enable_fanout) {
+        int fanout_arg = (fanout_group_id | (fanout_type << 16));
+
+        int setsockopt_fanout = setsockopt(packet_socket, SOL_PACKET, PACKET_FANOUT, &fanout_arg, sizeof(fanout_arg));
+
+        if (setsockopt_fanout < 0) {
+            logger << log4cpp::Priority::ERROR << "Can't configure fanout for interface " << interface_name
+                << " error number: " << errno << " error: " << strerror(errno);
+
+            return false;
+        }
+    }
+
+    // Start traffic collection loop
+    read_packets_from_socket(packet_socket, rd);
+
+    return true;
+}
+
+// Reads traffic from iovec using poll
+void read_packets_from_socket(int packet_socket, struct iovec* rd) {
+    unsigned int current_block_num = 0;
+
+    struct pollfd pfd;
+    memset(&pfd, 0, sizeof(pfd));
+
+    pfd.fd      = packet_socket;
+    pfd.events  = POLLIN | POLLERR;
+    pfd.revents = 0;
+
+    while (true) {
+        struct block_desc* pbd = (struct block_desc*)rd[current_block_num].iov_base;
+
+        if ((pbd->h1.block_status & TP_STATUS_USER) == 0) {
+            poll(&pfd, 1, -1);
+
+            continue;
+        }
+
+        blocks_read++;
+
+        walk_block(pbd);
+        flush_block(pbd);
+        current_block_num = (current_block_num + 1) % blocknum;
+    }
+
+    return;
+}
+
+void start_af_packet_capture(std::string interface_name, bool enable_fanout, int fanout_group_id) {
+    setup_socket(interface_name, enable_fanout, fanout_group_id);
+}
 
 void start_afpacket_collection(process_packet_pointer func_ptr) {
     logger << log4cpp::Priority::INFO << "AF_PACKET plugin started";
     afpacket_process_func_ptr = func_ptr;
 
-    unsigned int num_cpus = sysconf(_SC_NPROCESSORS_ONLN);
-    logger.info("We have %d cpus for AF_PACKET", num_cpus);
-
-    if (configuration_map.count("af_packet_read_packet_length_from_ip_header") != 0) {
-        afpacket_read_packet_length_from_ip_header = configuration_map["af_packet_read_packet_length_from_ip_header"] == "on";
-    }
-
-    std::string interfaces_list = "";
-
-    if (configuration_map.count("interfaces") != 0) {
-        interfaces_list = configuration_map["interfaces"];
-    }
-
+    // It's not compatible with Advanced and has no alternative
     if (configuration_map.count("mirror_af_packet_custom_sampling_rate") != 0) {
         mirror_af_packet_custom_sampling_rate =
             convert_string_to_integer(configuration_map["mirror_af_packet_custom_sampling_rate"]);
     }
 
-    if (configuration_map.count("mirror_af_packet_fanout_mode") != 0) {
-        // Set FANOUT mode
-        fanout_type = get_fanout_by_name(configuration_map["mirror_af_packet_fanout_mode"]);
-    }
+    // Set FANOUT mode
+    fanout_type = get_fanout_by_name(fastnetmon_global_configuration.mirror_af_packet_fanout_mode);
 
-    std::vector<std::string> interfaces_for_listen;
-    boost::split(interfaces_for_listen, interfaces_list, boost::is_any_of(","), boost::token_compress_on);
+    unsigned int num_cpus = sysconf(_SC_NPROCESSORS_ONLN);
+    logger.info("We have %d cpus for AF_PACKET", num_cpus);
 
-    logger << log4cpp::Priority::INFO << "AF_PACKET will listen on " << interfaces_for_listen.size() << " interfaces";
-
-    if (interfaces_for_listen.size() == 0) {
+    if (fastnetmon_global_configuration.interfaces.size() == 0) {
         logger << log4cpp::Priority::ERROR << "Please specify intreface for AF_PACKET";
         return;
     }
 
+    logger << log4cpp::Priority::DEBUG << "AF_PACKET will listen on "
+           << fastnetmon_global_configuration.interfaces.size() << " interfaces";
+
     // Thread group for all "master" processes
     boost::thread_group af_packet_main_threads;
 
-    for (std::vector<std::string>::size_type i = 0; i < interfaces_for_listen.size(); i++) {
+    for (std::vector<std::string>::size_type i = 0; i < fastnetmon_global_configuration.interfaces.size(); i++) {
         // Use process id to identify particular fanout group
         int group_identifier = getpid();
 
@@ -393,7 +434,7 @@ void start_afpacket_collection(process_packet_pointer func_ptr) {
 
         int fanout_group_id = group_identifier & 0xffff;
 
-        std::string capture_interface = interfaces_for_listen[i];
+        std::string capture_interface = fastnetmon_global_configuration.interfaces[i];
 
         logger << log4cpp::Priority::INFO << "AF_PACKET will listen on " << capture_interface << " interface";
 
@@ -423,7 +464,7 @@ void start_af_packet_capture_for_interface(std::string capture_interface, int fa
 
             boost::thread::attributes thread_attrs;
 
-            if (afpacket_execute_strict_cpu_affinity) {
+            if (fastnetmon_global_configuration.afpacket_execute_strict_cpu_affinity) {
                 cpu_set_t current_cpu_set;
 
                 int cpu_to_bind = cpu % num_cpus;
