@@ -34,6 +34,8 @@
 #include <boost/algorithm/string.hpp>
 #include <boost/regex.hpp>
 
+#include "iana_ip_protocols.hpp"
+
 boost::regex regular_expression_cidr_pattern("^\\d+\\.\\d+\\.\\d+\\.\\d+\\/\\d+$");
 boost::regex regular_expression_host_pattern("^\\d+\\.\\d+\\.\\d+\\.\\d+$");
 
@@ -144,7 +146,7 @@ std::string get_net_address_from_network_as_string(std::string network_cidr_form
     split(subnet_as_string, network_cidr_format, boost::is_any_of("/"), boost::token_compress_on);
 
     if (subnet_as_string.size() != 2) {
-        return 0;
+        return std::string();
     }
 
     return subnet_as_string[0];
@@ -477,8 +479,13 @@ std::string print_simple_packet(simple_packet_t packet) {
         // WTF?
     }
 
+    std::string protocol_name = get_ip_protocol_name_by_number_iana(packet.protocol);
+
+    // We use lowercase format
+    boost::algorithm::to_lower(protocol_name);
+
     buffer << source_ip_as_string << ":" << packet.source_port << " > " << destination_ip_as_string << ":"
-           << packet.destination_port << " protocol: " << get_printable_protocol_name(packet.protocol);
+           << packet.destination_port << " protocol: " << protocol_name;
 
     // Print flags only for TCP
     if (packet.protocol == IPPROTO_TCP) {
@@ -1790,13 +1797,14 @@ bool execute_web_request(std::string address,
 }
 
 // Write data to influxdb
-bool write_data_to_influxdb(std::string database,
-                            std::string host,
-                            std::string port,
+bool write_data_to_influxdb(const std::string& database,
+                            const std::string& host,
+                            const std::string& port,
                             bool enable_auth,
-                            std::string influx_user,
-                            std::string influx_password,
-                            std::string query) {
+                            const std::string& influx_user,
+                            const std::string& influx_password,
+                            const std::string& query,
+			    std::string& error_text) {
     uint32_t response_code = 0;
 
     std::string address = host + ":" + port;
@@ -1818,7 +1826,6 @@ bool write_data_to_influxdb(std::string database,
     std::string response_body;
 
     std::map<std::string, std::string> headers;
-    std::string error_text;
     bool result = execute_web_request(influxdb_query_string, "post", query, response_code, response_body, headers, error_text);
 
     if (!result) {
@@ -1826,6 +1833,7 @@ bool write_data_to_influxdb(std::string database,
     }
 
     if (response_code != 204) {
+	error_text = "Unexpected response code: " + std::to_string(response_code);
         return false;
     }
 
@@ -2192,28 +2200,30 @@ bool get_cpu_flags(std::vector<std::string>& flags) {
     return false;
 }
 
-std::string get_cpu_model() {
+bool get_cpu_model(std::string& cpu_model) {
     extern log4cpp::Category& logger;
 
     std::ifstream cpuinfo_file("/proc/cpuinfo");
-    boost::regex processor_model_pattern("^model name\\s+:\\s(.*?)$");
 
     if (!cpuinfo_file.is_open()) {
-        logger << log4cpp::Priority::ERROR << "License: could not open cpuinfo";
-        return "";
+        logger << log4cpp::Priority::ERROR << "Could not open /proc/cpuinfo";
+        return false;
     }
 
+    boost::regex processor_model_pattern("^model name\\s+:\\s(.*?)$");
     std::string line;
+
     while (getline(cpuinfo_file, line)) {
         boost::match_results<std::string::const_iterator> regex_results;
 
         if (boost::regex_match(line, regex_results, processor_model_pattern)) {
-            return regex_results[1];
+            cpu_model = regex_results[1];
+            return true;
         }
     }
 
-    // For new ARMs (Cavium Thunder X for example) we do not have model name in 4.10 kernel
-#ifdef __aarch64__
+    // For ARM CPUs we have another format
+    // Even if we run in x86_64 mode we can have cpuinfo with such information on ARM64 based macOS platforms
     std::string implementer;
     std::string part;
     std::string revision;
@@ -2246,13 +2256,13 @@ std::string get_cpu_model() {
 
     // If we fould all of them, use these fields as model
     if (implementer.size() > 0 && part.size() > 0 && revision.size() > 0) {
-        return "implementer: " + implementer + " part: " + part + " revision: " + revision;
-    } else {
-        // logger << log4cpp::Priority::ERROR << "implementer: " << implementer << " part: " << part << " revision: " << revision;
+        cpu_model = "implementer: " + implementer + " part: " + part + " revision: " + revision;
+        return true;
     }
-#endif
 
-    return "";
+    // logger << log4cpp::Priority::ERROR << "implementer: " << implementer << " part: " << part << " revision: " << revision;
+
+    return false;
 }
 
 // returns forwarding status as string
@@ -2465,3 +2475,39 @@ void print_simple_packet_buffer_to_string(const boost::circular_buffer<simple_pa
 }
 
 
+// Write circular buffer with simple packets to json document
+bool write_simple_packet_as_separate_fields_dump_to_json(const boost::circular_buffer<simple_packet_t>& simple_packets_buffer,
+                                                         nlohmann::json& packet_array) {
+    extern log4cpp::Category& logger;
+
+    // Even if we have no data we need empty array here
+    packet_array = nlohmann::json::array();
+
+    try {
+
+        if (simple_packets_buffer.size() == 0) {
+            logger << log4cpp::Priority::INFO << "Packet buffer is blank";
+            return true;
+        }
+
+        // Add all pack descriptions as strings array
+        for (const simple_packet_t& packet : simple_packets_buffer) {
+            nlohmann::json json_packet;
+
+            if (!serialize_simple_packet_to_json(packet, json_packet)) {
+                continue;
+            }
+
+            // Append to document as normal STL container
+            packet_array.push_back(json_packet);
+        }
+
+    } catch (...) {
+        logger << log4cpp::Priority::ERROR << "Cannot create packet list in JSON";
+        return false;
+    }
+
+    return true;
+}
+
+
