@@ -108,13 +108,14 @@ bool read_ipv6_host_from_string(std::string ipv6_host_as_string, in6_addr& resul
 bool validate_ipv6_or_ipv4_host(const std::string host);
 uint64_t get_current_unix_time_in_nanoseconds();
 
-bool write_data_to_influxdb(std::string database,
-                            std::string host,
-                            std::string port,
+bool write_data_to_influxdb(const std::string& database,
+                            const std::string& host,
+                            const std::string& port,
                             bool enable_auth,
-                            std::string influx_user,
-                            std::string influx_password,
-                            std::string query);
+                            const std::string& influx_user,
+                            const std::string& influx_password,
+                            const std::string& query,
+                            std::string& error_text);
 
 std::string join_by_comma_and_equal(const std::map<std::string, std::string>& data);
 bool parse_meminfo_into_map(std::map<std::string, uint64_t>& parsed_meminfo);
@@ -138,7 +139,7 @@ bool execute_web_request(const std::string& address_param,
                          const std::map<std::string, std::string>& headers,
                          std::string& error_text);
 unsigned int get_total_memory();
-std::string get_cpu_model();
+bool get_cpu_model(std::string& cpu_model);
 bool execute_web_request_secure(std::string address,
                                 std::string request_type,
                                 std::string post_data,
@@ -170,3 +171,5 @@ bool read_ipv6_subnet_from_string(subnet_ipv6_cidr_mask_t& ipv6_address, const s
 bool subnet_belongs_to_patricia_tree(patricia_tree_t* patricia_tree, const subnet_cidr_mask_t& subnet);
 // Prepares textual dump of simple packets buffer
 void print_simple_packet_buffer_to_string(const boost::circular_buffer<simple_packet_t>& simple_packets_buffer, std::string& output);
+bool write_simple_packet_as_separate_fields_dump_to_json(const boost::circular_buffer<simple_packet_t>& simple_packets_buffer,
+                                                         nlohmann::json& packet_array);
