@@ -30,7 +30,7 @@
 
 #include "abstract_subnet_counters.hpp"
 
-#include "fastnetmon.grpc.pb.h"
+#include "fastnetmon_internal_api.grpc.pb.h"
 #include <grpc++/grpc++.h>
 
 #ifdef __GNUC__
@@ -85,6 +85,8 @@
 
 #include <boost/program_options.hpp>
 
+#include "fastnetmon_configuration_scheme.hpp"
+
 #include "all_logcpp_libraries.hpp"
 
 // We do not have syslog.h on Windows
@@ -118,7 +120,7 @@
 #include "metrics/influxdb.hpp"
 
 // It's not enabled by default and we enable it only when we have Clickhouse libraries on platform
-#ifdef ENABLE_CLICKHOUSE_SUPPORT
+#ifdef CLICKHOUSE_SUPPORT
 #include "metrics/clickhouse.hpp"
 #endif
 
@@ -127,9 +129,9 @@
 #endif
 
 #ifdef FASTNETMON_API
-using fastmitigation::BanListReply;
-using fastmitigation::BanListRequest;
-using fastmitigation::Fastnetmon;
+
+#include "api.hpp"
+
 using grpc::Server;
 using grpc::ServerBuilder;
 using grpc::ServerContext;
@@ -143,6 +145,8 @@ bool enable_api = false;
 cppkafka::Producer* kafka_traffic_export_producer = nullptr;
 #endif
 
+fastnetmon_configuration_t fastnetmon_global_configuration;
+
 // Traffic export to Kafka
 bool kafka_traffic_export = false;
 
@@ -211,10 +215,6 @@ int unban_iteration_sleep_time = 60;
 
 bool unban_enabled = true;
 
-#ifdef ENABLE_GOBGP
-bool gobgp_enabled = false;
-#endif
-
 #ifdef MONGO
 std::string mongodb_host  = "localhost";
 unsigned int mongodb_port = 27017;
@@ -253,6 +253,12 @@ ban_settings_t global_ban_settings;
 // We use these flow spec rules as custom whitelist
 std::vector<flow_spec_rule_t> static_flowspec_based_whitelist;
 
+std::string graphite_thread_execution_time_desc = "Time consumed by pushing data to Graphite";
+struct timeval graphite_thread_execution_time;
+
+// Run stats thread
+bool usage_stats = true;
+
 void init_global_ban_settings() {
     // ban Configuration params
     global_ban_settings.enable_ban_for_pps              = false;
@@ -284,12 +290,9 @@ void init_global_ban_settings() {
 
 bool enable_connection_tracking = true;
 
-bool enable_afpacket_collection         = false;
 bool enable_af_xdp_collection           = false;
 bool enable_data_collection_from_mirror = false;
 bool enable_netmap_collection           = false;
-bool enable_sflow_collection            = false;
-bool enable_netflow_collection          = false;
 bool enable_pcap_collection             = false;
 
 std::string speed_calculation_time_desc = "Time consumed by recalculation for all IPs";
@@ -352,8 +355,13 @@ log4cpp::Category& logger = log4cpp::Category::getRoot();
 
 /* Configuration block ends */
 
-// We count total number of incoming/outgoing/internal and other traffic type packets/bytes
+// Total IPv4 + IPv6 traffic
+total_speed_counters_t total_counters;
+
+// Total IPv4 traffic
 total_speed_counters_t total_counters_ipv4;
+
+// Total IPv6 traffic
 total_speed_counters_t total_counters_ipv6;
 
 std::string total_unparsed_packets_desc = "Total number of packets we failed to parse";
@@ -382,8 +390,14 @@ uint64_t outgoing_total_flows_speed = 0;
 
 uint64_t total_flowspec_whitelist_packets         = 0;
 
+std::string clickhouse_metrics_writes_total_desc = "Total number of Clickhouse writes";
+uint64_t clickhouse_metrics_writes_total         = 0; 
+
+std::string clickhouse_metrics_writes_failed_desc = "Total number of failed Clickhouse writes";
+uint64_t clickhouse_metrics_writes_failed         = 0;
+
 // Network counters for IPv6
-abstract_subnet_counters_t<subnet_ipv6_cidr_mask_t, subnet_counter_t> ipv6_subnet_counters;
+abstract_subnet_counters_t<subnet_ipv6_cidr_mask_t, subnet_counter_t> ipv6_network_counters;
 
 // Host counters for IPv6
 abstract_subnet_counters_t<subnet_ipv6_cidr_mask_t, subnet_counter_t> ipv6_host_counters;
@@ -436,31 +450,12 @@ std::string exabgp_community_host   = "";
 
 std::string exabgp_next_hop     = "";
 
-// Graphite monitoring
-bool graphite_enabled             = false;
-std::string graphite_host         = "127.0.0.1";
-unsigned short int graphite_port  = 2003;
-unsigned int graphite_push_period = 1;
-
-// Default graphite namespace
-std::string graphite_prefix = "fastnetmon";
-
 std::string influxdb_writes_total_desc = "Total number of InfluxDB writes";
 uint64_t influxdb_writes_total         = 0;
 
 std::string influxdb_writes_failed_desc = "Total number of failed InfluxDB writes";
 uint64_t influxdb_writes_failed         = 0;
 
-// InfluxDB
-bool influxdb_enabled             = false;
-std::string influxdb_database     = "fastnetmon";
-std::string influxdb_host         = "127.0.0.1";
-unsigned short int influxdb_port  = 8086;
-bool influxdb_auth                = false;
-std::string influxdb_user         = "";
-std::string influxdb_password     = "";
-unsigned int influxdb_push_period = 1;
-
 bool process_incoming_traffic = true;
 bool process_outgoing_traffic = true;
 
@@ -472,10 +467,6 @@ void free_up_all_resources();
 void interruption_signal_handler(int signal_number);
 
 #ifdef FASTNETMON_API
-void silent_logging_function(gpr_log_func_args* args) {
-    // We do not want any logging here
-}
-
 // We could not define this variable in top of the file because we should define class before
 FastnetmonApiServiceImpl api_service;
 
@@ -677,7 +668,7 @@ bool load_configuration_file() {
     }
 
     if (configuration_map.count("graphite_prefix") != 0) {
-        graphite_prefix = configuration_map["graphite_prefix"];
+        fastnetmon_global_configuration.graphite_prefix = configuration_map["graphite_prefix"];
     }
 
     if (configuration_map.count("average_calculation_time") != 0) {
@@ -705,7 +696,7 @@ bool load_configuration_file() {
 #ifdef ENABLE_GOBGP
     // GoBGP configuration
     if (configuration_map.count("gobgp") != 0) {
-        gobgp_enabled = configuration_map["gobgp"] == "on";
+        fastnetmon_global_configuration.gobgp = configuration_map["gobgp"] == "on";
     }
 #endif
 
@@ -777,22 +768,92 @@ bool load_configuration_file() {
         }
     }
 
+    // sFlow section
     if (configuration_map.count("sflow") != 0) {
         if (configuration_map["sflow"] == "on") {
-            enable_sflow_collection = true;
+            fastnetmon_global_configuration.sflow = true;
         } else {
-            enable_sflow_collection = false;
+            fastnetmon_global_configuration.sflow = false;
         }
     }
 
+    if (configuration_map.count("sflow_host") != 0) {
+        fastnetmon_global_configuration.sflow_host = configuration_map["sflow_host"];
+    }
+
+    if (configuration_map.count("sflow_read_packet_length_from_ip_header") != 0) {
+        fastnetmon_global_configuration.sflow_read_packet_length_from_ip_header =
+	    configuration_map["sflow_read_packet_length_from_ip_header"] == "on";
+    }
+
+    // Read sFlow ports
+    std::string sflow_ports_string = "";
+
+    // Please note that it differs from field name in Advanced edition which uses "sflow_ports"
+    if (configuration_map.count("sflow_port") != 0) {
+        sflow_ports_string = configuration_map["sflow_port"];
+    }
+
+    std::vector<std::string> sflow_ports_for_listen;
+    boost::split(sflow_ports_for_listen, sflow_ports_string, boost::is_any_of(","), boost::token_compress_on);
+
+    std::vector<unsigned int> sflow_ports;
+
+    for (auto port_string : sflow_ports_for_listen) {
+        unsigned int sflow_port = convert_string_to_integer(port_string);
+
+        if (sflow_port == 0) {
+            logger << log4cpp::Priority::ERROR << "Cannot parse sFlow port: " << port_string;
+            continue;
+        }
+
+        fastnetmon_global_configuration.sflow_ports.push_back(sflow_port);
+    }
+
+    logger << log4cpp::Priority::INFO << "We parsed " << fastnetmon_global_configuration.sflow_ports.size() << " ports for sFlow";
+
+    // Netflow
     if (configuration_map.count("netflow") != 0) {
         if (configuration_map["netflow"] == "on") {
-            enable_netflow_collection = true;
+            fastnetmon_global_configuration.netflow = true;
         } else {
-            enable_netflow_collection = false;
+            fastnetmon_global_configuration.netflow = false;
         }
     }
 
+    if (configuration_map.count("netflow_host") != 0) {
+        fastnetmon_global_configuration.netflow_host = configuration_map["netflow_host"];
+    }
+
+    // Netflow ports
+    std::string netflow_ports_string = "";
+
+    if (configuration_map.count("netflow_port") != 0) {
+        netflow_ports_string = configuration_map["netflow_port"];
+    }
+
+    if (configuration_map.count("netflow_sampling_ratio") != 0) {
+        fastnetmon_global_configuration.netflow_sampling_ratio = convert_string_to_integer(configuration_map["netflow_sampling_ratio"]);
+
+        logger << log4cpp::Priority::INFO << "Using custom sampling ratio for Netflow v9 and IPFIX: " << fastnetmon_global_configuration.netflow_sampling_ratio;
+    }
+
+    std::vector<std::string> ports_for_listen;
+    boost::split(ports_for_listen, netflow_ports_string, boost::is_any_of(","), boost::token_compress_on);
+
+    std::vector<unsigned int> netflow_ports;
+
+    for (auto port : ports_for_listen) {
+        unsigned int netflow_port = convert_string_to_integer(port);
+
+        if (netflow_port == 0) {
+            logger << log4cpp::Priority::ERROR << "Cannot parse Netflow port: " << port;
+            continue;
+        }
+
+        fastnetmon_global_configuration.netflow_ports.push_back(netflow_port);
+    }
+
     if (configuration_map.count("exabgp_announce_whole_subnet") != 0) {
         exabgp_announce_whole_subnet = configuration_map["exabgp_announce_whole_subnet"] == "on" ? true : false;
     }
@@ -803,52 +864,81 @@ bool load_configuration_file() {
 
     // Graphite
     if (configuration_map.count("graphite") != 0) {
-        graphite_enabled = configuration_map["graphite"] == "on" ? true : false;
+        fastnetmon_global_configuration.graphite = configuration_map["graphite"] == "on" ? true : false;
     }
 
     if (configuration_map.count("graphite_host") != 0) {
-        graphite_host = configuration_map["graphite_host"];
+        fastnetmon_global_configuration.graphite_host = configuration_map["graphite_host"];
     }
 
     if (configuration_map.count("graphite_port") != 0) {
-        graphite_port = convert_string_to_integer(configuration_map["graphite_port"]);
+        fastnetmon_global_configuration.graphite_port = convert_string_to_integer(configuration_map["graphite_port"]);
     }
 
     if (configuration_map.count("graphite_push_period") != 0) {
-        graphite_push_period = convert_string_to_integer(configuration_map["graphite_push_period"]);
+        fastnetmon_global_configuration.graphite_push_period = convert_string_to_integer(configuration_map["graphite_push_period"]);
     }
 
     // InfluxDB
     if (configuration_map.count("influxdb") != 0) {
-        influxdb_enabled = configuration_map["influxdb"] == "on" ? true : false;
+        fastnetmon_global_configuration.influxdb = configuration_map["influxdb"] == "on" ? true : false;
     }
 
     if (configuration_map.count("influxdb_port") != 0) {
-        influxdb_port = convert_string_to_integer(configuration_map["influxdb_port"]);
+        fastnetmon_global_configuration.influxdb_port = convert_string_to_integer(configuration_map["influxdb_port"]);
     }
 
     if (configuration_map.count("influxdb_push_period") != 0) {
-        influxdb_push_period = convert_string_to_integer(configuration_map["influxdb_push_period"]);
+        fastnetmon_global_configuration.influxdb_push_period = convert_string_to_integer(configuration_map["influxdb_push_period"]);
     }
 
     if (configuration_map.count("influxdb_host") != 0) {
-        influxdb_host = configuration_map["influxdb_host"];
+        fastnetmon_global_configuration.influxdb_host = configuration_map["influxdb_host"];
     }
 
     if (configuration_map.count("influxdb_database") != 0) {
-        influxdb_database = configuration_map["influxdb_database"];
+        fastnetmon_global_configuration.influxdb_database = configuration_map["influxdb_database"];
     }
 
     if (configuration_map.count("influxdb_auth") != 0) {
-        influxdb_auth = configuration_map["influxdb_auth"] == "on" ? true : false;
+        fastnetmon_global_configuration.influxdb_auth = configuration_map["influxdb_auth"] == "on" ? true : false;
     }
 
     if (configuration_map.count("influxdb_user") != 0) {
-        influxdb_user = configuration_map["influxdb_user"];
+        fastnetmon_global_configuration.influxdb_user = configuration_map["influxdb_user"];
     }
 
     if (configuration_map.count("influxdb_password") != 0) {
-        influxdb_password = configuration_map["influxdb_password"];
+        fastnetmon_global_configuration.influxdb_password = configuration_map["influxdb_password"];
+    }
+
+    // Clickhouse
+    if (configuration_map.contains("clickhouse_metrics")) {
+        fastnetmon_global_configuration.clickhouse_metrics = configuration_map["clickhouse_metrics"] == "on" ? true : false;
+    }
+
+    if (configuration_map.contains("clickhouse_metrics_database")) {
+        fastnetmon_global_configuration.clickhouse_metrics_database = configuration_map["clickhouse_metrics_database"];
+    }
+
+    if (configuration_map.contains("clickhouse_metrics_username")) {
+        fastnetmon_global_configuration.clickhouse_metrics_username = configuration_map["clickhouse_metrics_username"];
+    }
+
+    if (configuration_map.contains("clickhouse_metrics_password")) {
+        fastnetmon_global_configuration.clickhouse_metrics_password = configuration_map["clickhouse_metrics_password"];
+    }
+
+    if (configuration_map.contains("clickhouse_metrics_host")) {
+        fastnetmon_global_configuration.clickhouse_metrics_host = configuration_map["clickhouse_metrics_host"];
+    }
+
+    if (configuration_map.contains("clickhouse_metrics_port") != 0) {
+        fastnetmon_global_configuration.clickhouse_metrics_port = convert_string_to_integer(configuration_map["clickhouse_metrics_port"]);
+    }
+
+    if (configuration_map.contains("clickhouse_metrics_push_period") != 0) {
+        fastnetmon_global_configuration.clickhouse_metrics_push_period = convert_string_to_integer(configuration_map["clickhouse_metrics_push_period"]);
     }
 
     if (configuration_map.count("process_incoming_traffic") != 0) {
@@ -883,11 +973,34 @@ bool load_configuration_file() {
         }
     }
 
+    // AF_PACKET Mirror
     if (configuration_map.count("mirror_afpacket") != 0) {
-        enable_afpacket_collection = configuration_map["mirror_afpacket"] == "on";
+        fastnetmon_global_configuration.mirror_afpacket = configuration_map["mirror_afpacket"] == "on";
     }
 
-    if (enable_afpacket_collection && enable_af_xdp_collection) {
+    std::string interfaces_list;
+    if (configuration_map.count("interfaces") != 0) {
+        interfaces_list = configuration_map["interfaces"];
+
+        std::vector<std::string> interfaces_for_listen;
+        boost::split(fastnetmon_global_configuration.interfaces, interfaces_list, boost::is_any_of(","), boost::token_compress_on);
+    }
+
+    // Please note that field name does not match name of configuration option 
+    if (configuration_map.count("af_packet_read_packet_length_from_ip_header") != 0) {
+        fastnetmon_global_configuration.af_packet_read_packet_length_from_ip_header =
+            configuration_map["af_packet_read_packet_length_from_ip_header"] == "on";
+    }
+
+    if (configuration_map.count("mirror_af_packet_fanout_mode") != 0) {
+        fastnetmon_global_configuration.mirror_af_packet_fanout_mode =
+            configuration_map["mirror_af_packet_fanout_mode"] == "on";
+    }
+
+
+    // XDP
+
+    if (fastnetmon_global_configuration.mirror_afpacket && enable_af_xdp_collection) {
         logger << log4cpp::Priority::ERROR << "You cannot use AF_XDP and AF_PACKET in same time, select one";
         exit(1);
     }
@@ -1750,10 +1863,6 @@ int main(int argc, char** argv) {
         std::filesystem::remove(fastnetmon_platform_configuration.backtrace_path);
     }
 
-#ifdef FASTNETMON_API
-    gpr_set_log_function(silent_logging_function);
-#endif
-
     // Set default ban configuration
     init_global_ban_settings();
 
@@ -1885,7 +1994,7 @@ int main(int argc, char** argv) {
 
     // We call init for each action
 #ifdef ENABLE_GOBGP
-    if (gobgp_enabled) {
+    if (fastnetmon_global_configuration.gobgp) {
         gobgp_action_init();
     }
 #endif
@@ -1946,9 +2055,6 @@ int main(int argc, char** argv) {
     set_boost_process_name(inaccurate_time_generator_thread, "fast_time");
     service_thread_group.add_thread(inaccurate_time_generator_thread);
 
-    // Run stats thread
-    bool usage_stats = true;
-
     if (configuration_map.count("disable_usage_report") != 0 && configuration_map["disable_usage_report"] == "on") {
         usage_stats = false;
     }
@@ -1966,15 +2072,23 @@ int main(int argc, char** argv) {
     service_thread_group.add_thread(new boost::thread(screen_draw_ipv6_thread));
 
     // Graphite export thread
-    if (graphite_enabled) {
+    if (fastnetmon_global_configuration.graphite) {
         service_thread_group.add_thread(new boost::thread(graphite_push_thread));
     }
 
     // InfluxDB export thread
-    if (influxdb_enabled) {
+    if (fastnetmon_global_configuration.influxdb) {
         service_thread_group.add_thread(new boost::thread(influxdb_push_thread));
     }
 
+#ifdef CLICKHOUSE_SUPPORT
+    // Clickhouse metrics export therad
+    if (fastnetmon_global_configuration.clickhouse_metrics) {
+	    logger << log4cpp::Priority::INFO << "Starting Clickhouse metrics export thread";
+        service_thread_group.add_thread(new boost::thread(clickhouse_push_thread));
+    }
+#endif
+
     // start thread for recalculating speed in realtime
     service_thread_group.add_thread(new boost::thread(recalculate_speed_thread_handler));
 
@@ -1996,7 +2110,7 @@ int main(int argc, char** argv) {
 #endif
 
 #ifdef FASTNETMON_ENABLE_AFPACKET
-    if (enable_afpacket_collection) {
+    if (fastnetmon_global_configuration.mirror_afpacket) {
         packet_capture_plugin_thread_group.add_thread(new boost::thread(start_afpacket_collection, process_packet));
     }
 #endif
@@ -2009,11 +2123,11 @@ int main(int argc, char** argv) {
     }
 #endif
 
-    if (enable_sflow_collection) {
+    if (fastnetmon_global_configuration.sflow) {
         packet_capture_plugin_thread_group.add_thread(new boost::thread(start_sflow_collection, process_packet));
     }
 
-    if (enable_netflow_collection) {
+    if (fastnetmon_global_configuration.netflow) {
         packet_capture_plugin_thread_group.add_thread(new boost::thread(start_netflow_collection, process_packet));
     }
 
