@@ -383,6 +383,85 @@ bool ipfix_record_to_flow(uint32_t record_type, uint32_t record_length, const ui
             packet.destination_port = fast_ntoh(packet.destination_port);
         }
 
+        break;
+    case IPFIX_TCP_SOURCE_PORT:
+        // This is unusual encoding used only by AMD Pensando
+        // We enable it only we know that packet is TCP
+        if (packet.protocol == IPPROTO_TCP) {
+
+            if (record_length == 2) {
+                uint16_t port = 0;
+                memcpy(&port, data, record_length);
+
+                packet.source_port = fast_ntoh(port);
+            } else {
+                ipfix_too_large_field++;
+
+                if (logger.getPriority() == log4cpp::Priority::DEBUG) {
+                    logger << log4cpp::Priority::DEBUG << "Unexpectedly big size for IPFIX_TCP_SOURCE_PORT: " << record_length;
+                }
+            }
+        }
+
+        break;
+    case IPFIX_TCP_DESTINATION_PORT:
+        // This is unusual encoding used only by AMD Pensando
+        // We enable it only we know that packet is TCP
+        if (packet.protocol == IPPROTO_TCP) {
+
+            if (record_length == 2) {
+                uint16_t port = 0;
+                memcpy(&port, data, record_length);
+
+                packet.destination_port = fast_ntoh(port);
+            } else {
+                ipfix_too_large_field++;
+
+                if (logger.getPriority() == log4cpp::Priority::DEBUG) {
+                    logger << log4cpp::Priority::DEBUG << "Unexpectedly big size for IPFIX_TCP_DESTINATION_PORT: " << record_length;
+                }
+            }
+        }
+
+        break;
+    case IPFIX_UDP_SOURCE_PORT:
+        // This is unusual encoding used only by AMD Pensando
+        // We enable it only we know that packet is UDP
+        if (packet.protocol == IPPROTO_UDP) {
+
+            if (record_length == 2) {
+                uint16_t port = 0;
+                memcpy(&port, data, record_length);
+
+                packet.source_port = fast_ntoh(port);
+            } else {
+                ipfix_too_large_field++;
+
+                if (logger.getPriority() == log4cpp::Priority::DEBUG) {
+                    logger << log4cpp::Priority::DEBUG << "Unexpectedly big size for IPFIX_UDP_SOURCE_PORT: " << record_length;
+                }
+            }
+        }
+
+        break;
+    case IPFIX_UDP_DESTINATION_PORT:
+        // This is unusual encoding used only by AMD Pensando
+        // We enable it only we know that packet is UDP
+        if (packet.protocol == IPPROTO_UDP) {
+
+            if (record_length == 2) {
+                uint16_t port = 0;
+                memcpy(&port, data, record_length);
+
+                packet.destination_port = fast_ntoh(port);
+            } else {
+                ipfix_too_large_field++;
+
+                if (logger.getPriority() == log4cpp::Priority::DEBUG) {
+                    logger << log4cpp::Priority::DEBUG << "Unexpectedly big size for IPFIX_UDP_DESTINATION_PORT: " << record_length;
+                }
+            }
+        }
         break;
     case IPFIX_IPV4_SRC_ADDR:
         if (record_length > sizeof(packet.src_ip)) {
@@ -748,6 +827,22 @@ bool ipfix_record_to_flow(uint32_t record_type, uint32_t record_length, const ui
             ipfix_forwarding_status++;
 
             // logger << log4cpp::Priority::DEBUG << "Forwarding status: " << int(forwarding_status_structure->status) << " reason code: " << int(forwarding_status_structure->reason_code);
+        } else if (record_length == 4) {
+            // We received 4 byte encoding from  Cisco ASR9006 running IOS XR 6.4.2
+            // It's new format which was added by RFC bugfix: https://datatracker.ietf.org/doc/draft-ietf-opsawg-ipfix-fixes/12/
+            // We still have only single byte with information but whole structure is larger
+            ipfix_forwarding_status_4_bytes_t forwarding_status{};
+
+            memcpy(&forwarding_status, data, record_length);
+
+            // Decode numbers into forwarding statuses
+            packet.forwarding_status             = forwarding_status_from_integer(forwarding_status.status);
+            flow_meta.received_forwarding_status = true;
+
+            ipfix_forwarding_status++;
+
+            // logger << log4cpp::Priority::DEBUG << "Forwarding status: " << int(forwarding_status.status) << " reason code: " << int(forwarding_status.reason_code);
+
         } else {
             // It must be exactly one byte
             ipfix_too_large_field++;
@@ -773,71 +868,72 @@ bool ipfix_record_to_flow(uint32_t record_type, uint32_t record_length, const ui
         }
 
         break;
-    case IPFIX_DATALINK_FRAME_SECTION:
+    case IPFIX_DATALINK_FRAME_SECTION: {
         // Element 315: https://www.iana.org/assignments/ipfix/ipfix.xhtml
 
         // It's packet header as is in variable length encoding
-        if (true) {
-            ipfix_inline_headers++;
+        ipfix_inline_headers++;
 
-            // This packet is ended using IPFIX variable length encoding and it may have two possible ways of length encoding
-            // https://datatracker.ietf.org/doc/html/rfc7011#section-7
-            if (flow_meta.variable_field_length_encoding == variable_length_encoding_t::single_byte || flow_meta.variable_field_length_encoding == variable_length_encoding_t::two_byte) {
+        // This packet is ended using IPFIX variable length encoding and it may have two possible ways of length
+        // encoding https://datatracker.ietf.org/doc/html/rfc7011#section-7
+        if (flow_meta.variable_field_length_encoding == variable_length_encoding_t::single_byte ||
+            flow_meta.variable_field_length_encoding == variable_length_encoding_t::two_byte) {
 
-                if (logger.getPriority() == log4cpp::Priority::DEBUG) {
-                    logger << log4cpp::Priority::DEBUG << "Packet header length: " << flow_meta.variable_field_length;
+            if (logger.getPriority() == log4cpp::Priority::DEBUG) {
+                logger << log4cpp::Priority::DEBUG << "Packet header length: " << flow_meta.variable_field_length;
+            }
+
+            if (flow_meta.variable_field_length != 0) {
+                bool read_packet_length_from_ip_header = true;
+
+                bool extract_tunnel_traffic = false;
+
+                const uint8_t* payload_shift = nullptr;
+
+                if (flow_meta.variable_field_length_encoding == variable_length_encoding_t::single_byte) {
+                    payload_shift = data + sizeof(uint8_t);
+                } else if (flow_meta.variable_field_length_encoding == variable_length_encoding_t::two_byte) {
+                    payload_shift = data + sizeof(uint8_t) + sizeof(uint16_t);
                 }
 
-                if (flow_meta.variable_field_length != 0) {
-                    bool read_packet_length_from_ip_header = true;
+                auto result =
+                    parse_raw_packet_to_simple_packet_full_ng(payload_shift, flow_meta.variable_field_length,
+                                                                flow_meta.variable_field_length, flow_meta.nested_packet,
+                                                              extract_tunnel_traffic, read_packet_length_from_ip_header);
 
-                    bool extract_tunnel_traffic = false;
-
-                    const uint8_t* payload_shift = nullptr;
-
-                    if (flow_meta.variable_field_length_encoding == variable_length_encoding_t::single_byte) {
-                        payload_shift = data + sizeof(uint8_t);
-                    } else if (flow_meta.variable_field_length_encoding == variable_length_encoding_t::two_byte) {
-                        payload_shift = data + sizeof(uint8_t) + sizeof(uint16_t);
-                    }
-
-                    auto result =
-                        parse_raw_packet_to_simple_packet_full_ng(payload_shift, flow_meta.variable_field_length,
-                                                                    flow_meta.variable_field_length, flow_meta.nested_packet,
-                                                                  extract_tunnel_traffic, read_packet_length_from_ip_header);
-
-                    if (result != network_data_stuctures::parser_code_t::success) {
-                        // Cannot decode data
-                        ipfix_inline_header_parser_error++;
-
-                        if (logger.getPriority() == log4cpp::Priority::DEBUG) {
-                            logger << log4cpp::Priority::DEBUG << "Cannot parse packet header with error: " << network_data_stuctures::parser_code_to_string(result);
-                        }
-
-                    } else {
-                        // Successfully decoded data
-                        ipfix_inline_header_parser_success++;
-
-                        flow_meta.nested_packet_parsed = true;
-                        // logger << log4cpp::Priority::DEBUG << "IPFIX inline extracted packet: " << print_simple_packet(flow_meta.nested_packet);
-                    }
-                } else {
-                    ipfix_inline_encoding_error++;
+                if (result != network_data_stuctures::parser_code_t::success) {
+                    // Cannot decode data
+                    ipfix_inline_header_parser_error++;
 
                     if (logger.getPriority() == log4cpp::Priority::DEBUG) {
-                        logger << log4cpp::Priority::DEBUG << "Zero length variable fields are not supported";
+                        logger << log4cpp::Priority::DEBUG << "Cannot parse packet header with error: "
+                            << network_data_stuctures::parser_code_to_string(result);
                     }
+
+                } else {
+                    // Successfully decoded data
+                    ipfix_inline_header_parser_success++;
+
+                    flow_meta.nested_packet_parsed = true;
+                    // logger << log4cpp::Priority::DEBUG << "IPFIX inline extracted packet: " << print_simple_packet(flow_meta.nested_packet);
                 }
             } else {
                 ipfix_inline_encoding_error++;
-                
+
                 if (logger.getPriority() == log4cpp::Priority::DEBUG) {
-                    logger << log4cpp::Priority::DEBUG << "Unknown variable field encoding type";
+                    logger << log4cpp::Priority::DEBUG << "Zero length variable fields are not supported";
                 }
             }
+        } else {
+            ipfix_inline_encoding_error++;
+            
+            if (logger.getPriority() == log4cpp::Priority::DEBUG) {
+                logger << log4cpp::Priority::DEBUG << "Unknown variable field encoding type";
+            }
         }
 
         break;
+    }
     case IPFIX_FLOW_DIRECTION:
         // It should be 1 byte value
         if (record_length == 1) {
@@ -917,7 +1013,8 @@ bool ipfix_record_to_flow(uint32_t record_type, uint32_t record_length, const ui
         //
         // TODO: this logic must be enabled via flag only as this is non RFC compliant behavior and we need to have confirmation from J
         //
-        // We have this guide from J: https://www.juniper.net/documentation/us/en/software/junos/flow-monitoring/topics/concept/services-ipfix-flow-aggregation-ipv6-extended-attributes.html but it's written in exceptionally weird way and raises more questions then answers
+        // We have this guide from J: https://www.juniper.net/documentation/us/en/software/junos/flow-monitoring/topics/concept/services-ipfix-flow-aggregation-ipv6-extended-attributes.html
+        // but it's written in exceptionally weird way and raises more questions then answers
         //
 
         // It's exactly 4 bytes
@@ -1145,6 +1242,7 @@ bool ipfix_flowset_to_store(const uint8_t* pkt,
                             ssize_t flowset_maximum_length,
                             const template_t* field_template,
                             uint32_t client_ipv4_address,
+                            uint32_t& flowset_length,
                             const std::string& client_addres_in_string_format) {
     simple_packet_t packet;
     packet.source       = NETFLOW;
@@ -1163,7 +1261,7 @@ bool ipfix_flowset_to_store(const uint8_t* pkt,
 
         if (itr == ipfix_sampling_rates.end()) {
             // Use global value
-            packet.sample_ratio = netflow_sampling_ratio;
+            packet.sample_ratio = fastnetmon_global_configuration.netflow_sampling_ratio;
         } else {
             packet.sample_ratio = itr->second;
         }
@@ -1223,7 +1321,7 @@ bool ipfix_flowset_to_store(const uint8_t* pkt,
                 flow_meta.variable_field_length = fast_ntoh(*two_byte_field_length_ptr);
 
                 // Override field length with length extracted from two bytes + length of placeholder byte itself
-                record_length = flow_meta.variable_field_length + sizeof(uint8_t) + sizeof(uint16_t);;
+                record_length = flow_meta.variable_field_length + sizeof(uint8_t) + sizeof(uint16_t);
 
                 // Pass variable payload length
                 flow_meta.variable_field_length = fast_ntoh(*two_byte_field_length_ptr);
@@ -1260,6 +1358,9 @@ bool ipfix_flowset_to_store(const uint8_t* pkt,
         offset += record_length;
     }
 
+    // At this moment offset carries full length of all fields
+    flowset_length = offset;
+
     // If we were able to decode nested packet then it means that it was Netflow Lite and we can overwrite information in packet
     if (flow_meta.nested_packet_parsed) {
         // Override most of the fields from nested packet as we need to use them instead
@@ -1404,6 +1505,7 @@ bool process_ipfix_data(const uint8_t* pkt,
                         uint32_t source_id,
                         const std::string& client_addres_in_string_format,
                         uint32_t client_ipv4_address) {
+    ipfix_total_packets++;
 
     const ipfix_data_flowset_header_t* flowset_header = (const ipfix_data_flowset_header_t*)pkt;
 
@@ -1442,6 +1544,8 @@ bool process_ipfix_data(const uint8_t* pkt,
     if (field_template->type == netflow_template_type_t::Data) {
 
         if (field_template->ipfix_variable_length_elements_used) {
+            // When we have variable length fields we need to use different logic which relies on flow length calculated during process of reading flow
+
             // Get clean flowsets length to use it as limit for our parser
             ssize_t current_flowset_length_no_header = flowset_length - sizeof(ipfix_data_flowset_header_t);
 
@@ -1449,12 +1553,48 @@ bool process_ipfix_data(const uint8_t* pkt,
                 logger << log4cpp::Priority::DEBUG << "IPFIX variable field element was used";
             }
 
-            // TODO: This implementation is rather limited as we read only single flowset here
-            // In many cases we use this stuff for IPFIX Inline monitoring and it uses just single flowset but it may change in future
-            // TODO: Juniper PTX uses multiple flowsets per packet
-            ipfix_flowset_to_store(pkt + sizeof(ipfix_data_flowset_header_t), ipfix_header, current_flowset_length_no_header,
-                                   field_template, client_ipv4_address, client_addres_in_string_format);
+            // Where all flows start in packet
+            const uint8_t* flow_section_start = pkt + sizeof(ipfix_data_flowset_header_t);
+
+            // Offset where flow starts
+            uint32_t flow_offset = 0;
+
+            // How much data we have in current flow set
+            uint32_t maximum_data_available_to_read = current_flowset_length_no_header;
+
+            // Run this loop until flow_offset reaches end of packet
+            while (flow_offset < current_flowset_length_no_header) {
+                // When variable fields present we need to read all fields before getting total length of flow
+                uint32_t read_flow_length = 0;
+
+                // In many cases we have just single flow per UDP packet but Juniper PTX uses multiple flows per packet
+                bool floset_processing_result =
+                    ipfix_flowset_to_store(flow_section_start + flow_offset, ipfix_header, maximum_data_available_to_read,
+                                           field_template, client_ipv4_address, read_flow_length, client_addres_in_string_format);
+
+                // If we cannot process this flowset then we must stop processing here because we need correct value of flowset_length to jump to next record
+                if (!floset_processing_result) {
+                    return false;
+                }
+
+                if (logger.getPriority() == log4cpp::Priority::DEBUG) {
+                    logger << log4cpp::Priority::DEBUG << "Total flow length: " << read_flow_length;
+                }
+
+                // Shift flowset offset by length of data read in this iteration
+                flow_offset += read_flow_length;
+
+                // And in the same time reduce amount of available to read data
+                maximum_data_available_to_read -= read_flow_length;
+            }
         } else {
+            // Check that template total length is not zero as we're going to divide by it
+            if (field_template->total_length == 0) {
+                logger << log4cpp::Priority::ERROR << "Zero IPFIX template length is not valid "
+                       << "client " << client_addres_in_string_format << " source_id: " << source_id;
+
+                return false;
+            }
 
             // This logic is pretty reliable but it works only if we do not have variable sized fields in template
             // In that case it's completely not applicable
@@ -1494,10 +1634,14 @@ bool process_ipfix_data(const uint8_t* pkt,
             }
 
             for (uint32_t i = 0; i < number_flowsets; i++) {
+                // We do not use it as we can use total_length directly instead of calculating it
+                uint32_t flowset_length = 0;
+
                 // We apply constraint that maximum potential length of flow set cannot exceed length of all fields in
                 // template In this case we have no fields with variable length which may affect it and we're safe
+                // We do not check response code as we can jump to next flow even if previous one failed
                 ipfix_flowset_to_store(pkt + offset, ipfix_header, field_template->total_length, field_template,
-                                       client_ipv4_address, client_addres_in_string_format);
+                                       client_ipv4_address, flowset_length, client_addres_in_string_format);
 
                 offset += field_template->total_length;
             }
@@ -1578,19 +1722,22 @@ bool process_ipfix_packet(const uint8_t* packet,
             return false;
         }
 
+        // Check that we have enough space in packet to read flowset header
+        if (offset + sizeof(ipfix_flowset_header_common_t) > ipfix_packet_length) {
+            logger << log4cpp::Priority::ERROR
+                   << "Flowset is too short: we do not have space for flowset header. "
+                   << "IPFIX packet agent IP:" << client_addres_in_string_format
+                   << " flowset number: " << flowset_number << " offset: " << offset << " packet_length: " << ipfix_packet_length;
+            return false;
+        }
+
         const ipfix_flowset_header_common_t* flowset = (const ipfix_flowset_header_common_t*)(packet + offset);
 
         uint32_t flowset_id     = ntohs(flowset->flowset_id);
         uint32_t flowset_length = ntohs(flowset->length);
 
-        /*
-         * Yes, this is a near duplicate of the short packet check
-         * above, but this one validates the flowset length from in
-         * the packet before we pass it to the flowset-specific
-         * handlers below.
-         */
-
-        if (offset + flowset_length > ipfix_packet_length) {
+        // One more check to ensure that we have enough space in packet to read whole flowset
+	    if (offset + flowset_length > ipfix_packet_length) {
             logger << log4cpp::Priority::ERROR
                    << "We tried to read from address outside IPFIX packet flowset agent IP: " << client_addres_in_string_format;
             return false;
