@@ -64,7 +64,9 @@ bool process_netflow_v9_options_template(const uint8_t* pkt, size_t flowset_leng
     std::vector<template_record_t> template_records_map;
     uint32_t total_size = 0;
 
-    for (; offset < fast_ntoh(options_nested_header->option_length);) {
+    uint32_t option_length = fast_ntoh(options_nested_header->option_length);
+
+    for (; offset < option_length;) {
         records_number++;
         const netflow9_template_flowset_record_t* tmplr =
             (const netflow9_template_flowset_record_t*)(zone_address_without_skopes + offset);
@@ -92,6 +94,15 @@ bool process_netflow_v9_options_template(const uint8_t* pkt, size_t flowset_leng
     field_template.type                = netflow_template_type_t::Options;
     field_template.option_scope_length = scopes_total_size;
 
+    // Templates with total length which is zero do not make any sense and have to be ignored
+    // We need templates to decode data blob and decoding zero length value is meaningless
+    if (field_template.total_length == 0) {
+        logger << log4cpp::Priority::ERROR
+               << "Received zero length malformed options Netfow v9 template " << template_id
+               << " from " << client_addres_in_string_format;
+        return false;
+    }
+
     // We need to know when we received it
     field_template.timestamp = current_inaccurate_time;
 
@@ -183,6 +194,15 @@ bool process_netflow_v9_template(const uint8_t* pkt,
             // TODO: introduce netflow9_check_rec_len
         }
 
+        // Templates with total length which is zero do not make any sense and have to be ignored
+        // We need templates to decode data blob and decoding zero length value is meaningless
+        if (total_size == 0) {
+            logger << log4cpp::Priority::ERROR
+                << "Received zero length malformed data Netflow v9 template " << template_id
+                << " from " << client_addres_in_string_format;
+            return false;
+        }
+
         template_t field_template{};
 
         field_template.template_id  = template_id;
@@ -617,41 +637,40 @@ bool netflow9_record_to_flow(uint32_t record_type,
         }
 
         break;
-    case NETFLOW9_LAYER2_PACKET_SECTION_DATA:
-        if (true) {
-            netflow_v9_lite_headers++;
+    case NETFLOW9_LAYER2_PACKET_SECTION_DATA: {
+        netflow_v9_lite_headers++;
 
-            bool read_packet_length_from_ip_header = true;
+        bool read_packet_length_from_ip_header = true;
 
-            // It's our safe fallback
-            uint64_t full_packet_length = record_length;
+        // It's our safe fallback
+        uint64_t full_packet_length = record_length;
 
-            // Device must provide this information on previous iteration, let's try to get it in case if we've got it:
-            if (flow_meta.data_link_frame_size != 0) {
-                full_packet_length = flow_meta.data_link_frame_size;
-            }
+        // Device must provide this information on previous iteration, let's try to get it in case if we've got it:
+        if (flow_meta.data_link_frame_size != 0) {
+            full_packet_length = flow_meta.data_link_frame_size;
+        }
 
-            bool extract_tunnel_traffic = false;
+        bool extract_tunnel_traffic = false;
 
-            auto result = parse_raw_packet_to_simple_packet_full_ng((u_char*)(data), full_packet_length, record_length,
-                                                                    flow_meta.nested_packet, extract_tunnel_traffic,
-                                                                    read_packet_length_from_ip_header);
+        auto result = parse_raw_packet_to_simple_packet_full_ng((u_char*)(data), full_packet_length, record_length,
+            flow_meta.nested_packet, extract_tunnel_traffic,
+            read_packet_length_from_ip_header);
 
-            if (result != network_data_stuctures::parser_code_t::success) {
-                // Cannot decode data
-                netflow_v9_lite_header_parser_error++;
+        if (result != network_data_stuctures::parser_code_t::success) {
+            // Cannot decode data
+            netflow_v9_lite_header_parser_error++;
 
-                if (logger.getPriority() == log4cpp::Priority::DEBUG) {
-                    logger << log4cpp::Priority::DEBUG << "Cannot parse packet header with error: " << network_data_stuctures::parser_code_to_string(result); 
-                }
-            } else {
-                netflow_v9_lite_header_parser_success++;
-                // Successfully decoded data
-                flow_meta.nested_packet_parsed = true;
+            if (logger.getPriority() == log4cpp::Priority::DEBUG) {
+                logger << log4cpp::Priority::DEBUG << "Cannot parse packet header with error: " << network_data_stuctures::parser_code_to_string(result); 
             }
+        } else {
+            netflow_v9_lite_header_parser_success++;
+            // Successfully decoded data
+            flow_meta.nested_packet_parsed = true;
         }
 
         break;
+    }
     // There is a similar field NETFLOW9_BGP_NEXT_HOP_IPV4_ADDRESS but with slightly different meaning
     // https://www.cisco.com/en/US/technologies/tk648/tk362/technologies_white_paper09186a00800a3db9.html
     case NETFLOW9_IPV4_NEXT_HOP:
@@ -712,6 +731,14 @@ bool netflow9_record_to_flow(uint32_t record_type,
 
             sampler_id = fast_ntoh(sampler_id);
 
+            // logger << log4cpp::Priority::DEBUG << "Got sampler id from data template: " << int(sampler_id);
+        } else if (record_length == 4) {
+            uint32_t sampler_id = 0;
+
+            memcpy(&sampler_id, data, record_length);
+
+            sampler_id = fast_ntoh(sampler_id);
+
             // logger << log4cpp::Priority::DEBUG << "Got sampler id from data template: " << int(sampler_id);
         } else {
             netflow_v9_too_large_field++;
@@ -1252,7 +1279,7 @@ void netflow9_flowset_to_store(const uint8_t* pkt,
 
         if (itr == netflow9_sampling_rates.end()) {
             // Use global value
-            packet.sample_ratio = netflow_sampling_ratio;
+            packet.sample_ratio = fastnetmon_global_configuration.netflow_sampling_ratio;
         } else {
             packet.sample_ratio = itr->second;
         }
@@ -1473,6 +1500,14 @@ bool process_netflow_v9_data(const uint8_t* pkt,
         return false;
     }
 
+    // Check that template total length is not zero as we're going to divide by it
+    if (field_template->total_length == 0) {
+        logger << log4cpp::Priority::ERROR
+               << "Zero template length is not valid "
+               << "client " << client_addres_in_string_format << " source_id: " << source_id;
+       return false;
+    }
+
     uint32_t offset       = sizeof(*dath);
     uint32_t num_flowsets = (flowset_length - offset) / field_template->total_length;
 
@@ -1552,18 +1587,22 @@ bool process_netflow_packet_v9(const uint8_t* packet,
             return false;
         }
 
+        // Check that we have enough space in packet to read flowset header
+        if (offset + sizeof(netflow9_flowset_header_common_t) > packet_length) {
+            logger << log4cpp::Priority::ERROR
+                   << "Flowset is too short: we do not have space for flowset header. "
+                   << "Netflow v9 packet agent IP:" << client_addres_in_string_format
+                   << " flowset number: " << flowset_number << " offset: " << offset << " packet_length: " << packet_length;
+            return false;
+        }
+
+        // Now we can safely read flowset header
         const netflow9_flowset_header_common_t* flowset = (const netflow9_flowset_header_common_t*)(packet + offset);
 
         uint32_t flowset_id     = ntohs(flowset->flowset_id);
         uint32_t flowset_length = ntohs(flowset->length);
 
-        /*
-         * Yes, this is a near duplicate of the short packet check
-         * above, but this one validates the flowset length from in
-         * the packet before we pass it to the flowset-specific
-         * handlers below.
-         */
-
+        // One more check to ensure that we have enough space in packet to read whole flowset
         if (offset + flowset_length > packet_length) {
             logger << log4cpp::Priority::ERROR << "We tried to read from address outside Netflow's packet flowset agent IP: "
                    << client_addres_in_string_format << " flowset number: " << flowset_number
