@@ -1,5 +1,4 @@
-#ifndef PCAP_PLUGIN_H
-#define PCAP_PLUGIN_H
+#pragma once
 
 #include "../fastnetmon_types.hpp"
 #include <iostream>
@@ -7,5 +6,3 @@
 void start_pcap_collection(process_packet_pointer func_ptr);
 void stop_pcap_collection();
 std::string get_pcap_stats();
-
-#endif
