@@ -138,6 +138,17 @@ sub get_library_binary_build_from_google_storage {
         return 0;
     }
 
+    # print "Start sha-512 calculation\n";
+    # Calculate hash of downloaded file
+    my $sha512 = get_sha_512_sum("/tmp/$dependency_archive_name");
+
+    unless ($sha512) {
+        warn "Cannot calculate SHA512 for file from S3\n";
+        return 2;
+    }
+
+    # print "Calculated sha-512 for $dependency_name $sha512\n";
+
     # Hashes for all distros
     my $data_hashes = shift;
 
@@ -152,20 +163,10 @@ sub get_library_binary_build_from_google_storage {
 
     # Hash must exist for all our existing dependencies
     unless ($current_build_hash) {
-        warn "Cannot get $dependency_name hash for Distro $distro_type $distro_version architecture $machine_architecture, please add it to build configuration";
+        warn "Cannot get $dependency_name hash for Distro $distro_type $distro_version architecture $machine_architecture, please add it to build configuration. Retrieved file with hash: $sha512";
         return 2;
     }
 
-    #print "Start sha-512 calculation\n";
-    my $sha512 = get_sha_512_sum("/tmp/$dependency_archive_name");
-
-    unless ($sha512) {
-        warn "Cannot calculate SHA512 for file from S3\n";
-        return 2;
-    }
-
-    # print "Calculated sha-512 for $dependency_name $sha512\n";
-
     if ($sha512 ne $current_build_hash) {
         warn "Hash mismatch. Expected: $current_build_hash got: $sha512. It may be sign of data tampering, please validate data source\n";
         return 2;
@@ -491,7 +492,7 @@ sub install_bpf {
     }
 
     my $elfutils_install_path = "$library_install_folder/elfutils_0_186";
-    my $zlib_path             = "$library_install_folder/zlib_1_2_13";
+    my $zlib_path             = "$library_install_folder/zlib_1_3_1";
 
     my $archive_file_name = 'v1.0.1.tar.gz ';
 
@@ -830,8 +831,8 @@ sub install_cares {
 sub install_zlib {
     my $folder_name = shift;
 
-    my $res = install_configure_based_software("https://zlib.net/zlib-1.2.13.tar.gz",
-        "55eaa84906f31ac20d725aa26cd20839196b6ba6", "$library_install_folder/$folder_name", "");
+    my $res = install_configure_based_software("https://zlib.net/zlib-1.3.1.tar.gz",
+        "f535367b1a11e2f9ac3bec723fb007fbc0d189e5", "$library_install_folder/$folder_name", "");
 
     unless ($res) {
         warn "Cannot install zlib\n";
@@ -866,13 +867,13 @@ sub install_grpc {
 
     my $protobuf_install_path = "$library_install_folder/protobuf_21_12";
 
-    my $abseil_install_path = "$library_install_folder/abseil_2022_06_23";
+    my $abseil_install_path = "$library_install_folder/abseil_2024_01_16";
 
     my $openssl_path = "$library_install_folder/$openssl_folder_name";
 
     my $cares_path = "$library_install_folder/cares_1_18_1";
 
-    my $zlib_path = "$library_install_folder/zlib_1_2_13";
+    my $zlib_path = "$library_install_folder/zlib_1_3_1";
 
     my $re2_path = "$library_install_folder/re2_2022_12_01";
 
@@ -1021,6 +1022,20 @@ sub install_rdkafka {
     return 1;
 }
 
+sub install_clickhouse {
+    my $folder_name = shift;
+
+    my $clickhouse_install_path = "$library_install_folder/$folder_name";
+
+    my $res = install_cmake_based_software("https://github.com/ClickHouse/clickhouse-cpp/archive/refs/tags/v2.3.0.tar.gz", "08a4a2e45ddcb33941c22f5e02177805d7fdd664",
+        $clickhouse_install_path, "$ld_library_path_for_make $cmake_path -DCMAKE_INSTALL_PREFIX:STRING=$clickhouse_install_path -DCMAKE_C_COMPILER=$default_c_compiler_path -DCMAKE_CXX_COMPILER=$default_cpp_compiler_path -DBUILD_SHARED_LIBS:BOOL=ON ..");
+
+    if (!$res) {
+        die "Could not install libclickhouse\n";
+    }
+
+    1;
+}
 
 
 sub install_elfutils {
@@ -1071,10 +1086,10 @@ sub install_abseil {
     # We need explicitly enable PIC to successfully build against gRPC
     # https://github.com/abseil/abseil-cpp/pull/741
     # -DCMAKE_POSITION_INDEPENDENT_CODE=true
-    my $res = install_cmake_based_software("https://github.com/abseil/abseil-cpp/archive/refs/tags/20220623.0.tar.gz",
-        "144c2108e1532c642cdb6ca532ee26e91146cf28",
+    my $res = install_cmake_based_software("https://github.com/abseil/abseil-cpp/archive/refs/tags/20240116.2.tar.gz",
+        "bb8a766f3aef8e294a864104b8ff3fc37b393210",
         "$library_install_folder/$folder_name",
-        "$ld_library_path_for_make $cmake_path -DCMAKE_C_COMPILER=$default_c_compiler_path -DCMAKE_CXX_COMPILER=$default_cpp_compiler_path -DABSL_BUILD_TESTING=ON -DABSL_USE_GOOGLETEST_HEAD=ON -DABSL_ENABLE_INSTALL=ON -DCMAKE_CXX_STANDARD=11 -DCMAKE_POSITION_INDEPENDENT_CODE=true -DCMAKE_INSTALL_PREFIX=$install_path ..");
+        "$ld_library_path_for_make $cmake_path -DCMAKE_C_COMPILER=$default_c_compiler_path -DCMAKE_CXX_COMPILER=$default_cpp_compiler_path -DABSL_BUILD_TESTING=OFF -DABSL_USE_GOOGLETEST_HEAD=ON -DABSL_ENABLE_INSTALL=ON -DCMAKE_CXX_STANDARD=14 -DCMAKE_POSITION_INDEPENDENT_CODE=true -DCMAKE_INSTALL_PREFIX=$install_path ..");
 
     if (!$res) {
         warn "Cannot install abseil\n";
