@@ -28,15 +28,13 @@ extern log4cpp::Category& logger;
 
 #include <boost/algorithm/string.hpp>
 
-// Global configuration map
-extern std::map<std::string, std::string> configuration_map;
+#include "../fastnetmon_configuration_scheme.hpp"
+
+extern fastnetmon_configuration_t fastnetmon_global_configuration;
 
 std::string raw_udp_packets_received_desc = "Number of raw packets received without any errors";
 uint64_t raw_udp_packets_received         = 0;
 
-// We have an option to use IP length from the packet header because some vendors may lie about it: https://github.com/pavel-odintsov/fastnetmon/issues/893
-bool sflow_read_packet_length_from_ip_header = false;
-
 std::string udp_receive_errors_desc = "Number of failed receives";
 uint64_t udp_receive_errors         = 0;
 
@@ -72,6 +70,13 @@ uint64_t sflow_parse_error_nested_header = 0;
 std::string sflow_counter_sample_desc = "Number of counter samples, i.e. with port counters";
 uint64_t sflow_counter_sample         = 0;
 
+std::string sflow_expanded_counter_sample_desc = "Number of expanded counter samples, i.e. with port counters";
+uint64_t sflow_expanded_counter_sample         = 0;
+
+std::string sflow_generic_interface_counter_sample_desc =
+    "Number of counter samples with generic interface counter information";
+uint64_t sflow_generic_interface_counter_sample = 0;
+
 std::string sflow_raw_packet_headers_total_desc = "Number of packet headers from flow samples";
 uint64_t sflow_raw_packet_headers_total         = 0;
 
@@ -93,6 +98,9 @@ uint64_t sflow_ipv4_header_protocol         = 0;
 std::string sflow_ipv6_header_protocol_desc = "Number of samples with IPv6 packet headers";
 uint64_t sflow_ipv6_header_protocol         = 0;
 
+std::string sflow_packets_discarded_desc = "Number of packets discarded by device";
+uint64_t sflow_packets_discarded         = 0;
+
 std::vector<system_counter_t> get_sflow_stats() {
     std::vector<system_counter_t> counters;
 
@@ -112,6 +120,13 @@ std::vector<system_counter_t> get_sflow_stats() {
     counters.push_back(system_counter_t("sflow_parse_error_nested_header", sflow_parse_error_nested_header,
                                         metric_type_t::counter, sflow_parse_error_nested_header_desc));
     counters.push_back(system_counter_t("sflow_counter_sample", sflow_counter_sample, metric_type_t::counter, sflow_counter_sample_desc));
+    
+    counters.push_back(system_counter_t("sflow_expanded_counter_sample", sflow_expanded_counter_sample,
+                                        metric_type_t::counter, sflow_expanded_counter_sample_desc));
+
+    counters.push_back(system_counter_t("sflow_generic_interface_counter_sample", sflow_generic_interface_counter_sample,
+                                        metric_type_t::counter, sflow_generic_interface_counter_sample_desc));
+
     counters.push_back(system_counter_t("sflow_raw_packet_headers_total", sflow_raw_packet_headers_total,
                                         metric_type_t::counter, sflow_raw_packet_headers_total_desc));
     counters.push_back(system_counter_t("sflow_ipv4_header_protocol", sflow_ipv4_header_protocol,
@@ -127,12 +142,16 @@ std::vector<system_counter_t> get_sflow_stats() {
     counters.push_back(system_counter_t("sflow_extended_gateway_data_records", sflow_extended_gateway_data_records,
                                         metric_type_t::counter, sflow_extended_gateway_data_records_desc));
 
+    counters.push_back(system_counter_t("sflow_packets_discarded", sflow_packets_discarded, metric_type_t::counter,
+                                        sflow_packets_discarded_desc));
+
+
     return counters;
 }
 
 // Prototypes
 
-bool process_sflow_counter_sample(uint8_t* data_pointer,
+bool process_sflow_counter_sample(const uint8_t* data_pointer,
                                   size_t data_length,
                                   bool expanded,
                                   const sflow_packet_header_unified_accessor& sflow_header_accessor);
@@ -144,7 +163,7 @@ void start_sflow_collector(const std::string& sflow_host, unsigned int sflow_por
 void init_sflow_module() {
 }
 
-// Deinitilize sflow module, we need it for deallocation module structures
+// De-initialise sflow module, we need it for deallocation module structures
 void deinit_sflow_module() {
 }
 
@@ -152,52 +171,23 @@ void start_sflow_collection(process_packet_pointer func_ptr) {
     logger << log4cpp::Priority::INFO << plugin_log_prefix << "plugin started";
     sflow_process_func_ptr = func_ptr;
 
-
-    std::string sflow_ports_string = "";
-
-    if (configuration_map.count("sflow_port") != 0) {
-        sflow_ports_string = configuration_map["sflow_port"];
-    }
-
-    std::vector<std::string> sflow_ports_for_listen;
-    boost::split(sflow_ports_for_listen, sflow_ports_string, boost::is_any_of(","), boost::token_compress_on);
-
-    std::vector<unsigned int> sflow_ports;
-
-    for (auto port_string : sflow_ports_for_listen) {
-        unsigned int sflow_port = convert_string_to_integer(port_string);
-
-        if (sflow_port == 0) {
-            logger << log4cpp::Priority::ERROR << plugin_log_prefix << "Cannot parse port: " << port_string;
-            continue;
-        }
-
-        sflow_ports.push_back(sflow_port);
-    }
-
-    if (sflow_ports.size() == 0) {
-        logger << log4cpp::Priority::ERROR << plugin_log_prefix << "Please specify least single port for sflow_port field!";
+    if (fastnetmon_global_configuration.sflow_ports.size() == 0) {
+        logger << log4cpp::Priority::ERROR << plugin_log_prefix << "Please specify at least single port for sflow_port field";
         return;
     }
 
-    logger << log4cpp::Priority::INFO << plugin_log_prefix << "We parsed " << sflow_ports.size() << " ports for sflow";
+    logger << log4cpp::Priority::DEBUG << plugin_log_prefix << "We parsed "
+           << fastnetmon_global_configuration.sflow_ports.size() << " ports for sFlow";
+
 
     boost::thread_group sflow_collector_threads;
 
-    logger << log4cpp::Priority::INFO << plugin_log_prefix << "We will listen on " << sflow_ports.size() << " ports";
+    logger << log4cpp::Priority::DEBUG << plugin_log_prefix << "We will listen on "
+           << fastnetmon_global_configuration.sflow_ports.size() << " ports";
 
-    std::string sflow_host;
-
-    if (configuration_map.count("sflow_host") != 0) {
-        sflow_host = configuration_map["sflow_host"];
-    }
-
-    if (configuration_map.count("sflow_read_packet_length_from_ip_header") != 0) {
-        sflow_read_packet_length_from_ip_header = configuration_map["sflow_read_packet_length_from_ip_header"] == "on";
-    }
-
-    for (auto sflow_port : sflow_ports) {
-        sflow_collector_threads.add_thread(new boost::thread(start_sflow_collector, sflow_host, sflow_port));
+    for (auto sflow_port : fastnetmon_global_configuration.sflow_ports) {
+        sflow_collector_threads.add_thread(
+            new boost::thread(start_sflow_collector, fastnetmon_global_configuration.sflow_host, sflow_port));
     }
 
     sflow_collector_threads.join_all();
@@ -220,7 +210,9 @@ void start_sflow_collector(const std::string& sflow_host, unsigned int sflow_por
 
     addrinfo* servinfo = NULL;
 
-    int getaddrinfo_result = getaddrinfo(sflow_host.c_str(), std::to_string(sflow_port).c_str(), &hints, &servinfo);
+    std::string port_as_string = std::to_string(sflow_port);
+
+    int getaddrinfo_result = getaddrinfo(sflow_host.c_str(), port_as_string.c_str(), &hints, &servinfo);
 
     if (getaddrinfo_result != 0) {
         logger << log4cpp::Priority::ERROR << "sFlow getaddrinfo function failed with code: " << getaddrinfo_result
@@ -238,16 +230,13 @@ void start_sflow_collector(const std::string& sflow_host, unsigned int sflow_por
     int bind_result = bind(sockfd, servinfo->ai_addr, servinfo->ai_addrlen);
 
     if (bind_result != 0) {
-        logger << log4cpp::Priority::ERROR << plugin_log_prefix << "cannot bind on " << sflow_port << ":"
-               << sflow_host << " with errno: " << errno << " error: " << strerror(errno);
+        logger << log4cpp::Priority::ERROR << plugin_log_prefix << "cannot bind on " << sflow_port << ":" << sflow_host
+               << " with errno: " << errno << " error: " << strerror(errno);
         return;
     }
 
     freeaddrinfo(servinfo);
 
-    struct sockaddr_in6 peer;
-    memset(&peer, 0, sizeof(peer));
-
     /* We should specify timeout there for correct toolkit shutdown */
     /* Because otherwise recvfrom will stay in blocked mode forever */
     struct timeval tv;
@@ -288,6 +277,9 @@ void start_sflow_collector(const std::string& sflow_host, unsigned int sflow_por
 
             if (client_addr.sin_family == AF_INET) {
                 client_ipv4_address = client_addr.sin_addr.s_addr;
+                // Use most efficient function to avoid bottlenecks here
+                std::string ip_address_as_string;
+                // convert_ip_as_uint_to_string_fast(client_ipv4_address, ip_address_as_string);
                 // logger << log4cpp::Priority::ERROR << "client ip: " << convert_ip_as_uint_to_string(client_ipv4_address);
             } else if (client_addr.sin_family == AF_INET6) {
                 // We do not support them now
@@ -314,12 +306,12 @@ void start_sflow_collector(const std::string& sflow_host, unsigned int sflow_por
     }
 }
 
-bool process_sflow_flow_sample(uint8_t* data_pointer,
+bool process_sflow_flow_sample(const uint8_t* data_pointer,
                                size_t data_length,
                                bool expanded,
                                const sflow_packet_header_unified_accessor& sflow_header_accessor,
                                uint32_t client_ipv4_address) {
-    uint8_t* current_packet_end = data_pointer + data_length;
+    const uint8_t* current_packet_end = data_pointer + data_length;
 
     sflow_sample_header_unified_accessor_t sflow_sample_header_unified_accessor;
 
@@ -338,7 +330,18 @@ bool process_sflow_flow_sample(uint8_t* data_pointer,
         return false;
     }
 
-    uint8_t* flow_record_zone_start = data_pointer + sflow_sample_header_unified_accessor.get_original_payload_length();
+    if (sflow_sample_header_unified_accessor.get_number_of_flow_records() > max_number_of_flow_records) {
+        logger << log4cpp::Priority::ERROR << plugin_log_prefix << "flow records number "
+            << sflow_sample_header_unified_accessor.get_number_of_flow_records()
+            << " exceeds maximum value "
+            << max_number_of_flow_records;
+
+        sflow_bad_flow_samples++;
+
+        return false;
+    }
+
+    const uint8_t* flow_record_zone_start = data_pointer + sflow_sample_header_unified_accessor.get_original_payload_length();
 
     std::vector<record_tuple_t> vector_tuple;
     vector_tuple.reserve(sflow_sample_header_unified_accessor.get_number_of_flow_records());
@@ -367,9 +370,9 @@ bool process_sflow_flow_sample(uint8_t* data_pointer,
     packet.agent_ip_address = client_ipv4_address;
 
     for (auto record : vector_tuple) {
-        int32_t record_type   = std::get<0>(record);
-        uint8_t* payload_ptr  = std::get<1>(record);
-        int32_t record_length = std::get<2>(record);
+        int32_t record_type        = std::get<0>(record);
+        const uint8_t* payload_ptr = std::get<1>(record);
+        int32_t record_length      = std::get<2>(record);
 
         // std::cout << "flow record " << " record_type: " << record_type
         //    << " record_length: " << record_length << std::endl;
@@ -384,17 +387,15 @@ bool process_sflow_flow_sample(uint8_t* data_pointer,
             sflow_raw_protocol_header.network_to_host_byte_order();
             // logger << log4cpp::Priority::DEBUG << "Raw protocol header: " << sflow_raw_protocol_header.print();
 
-            uint8_t* header_payload_pointer = payload_ptr + sizeof(sflow_raw_protocol_header_t);
+            const uint8_t* header_payload_pointer = payload_ptr + sizeof(sflow_raw_protocol_header_t);
 
             if (sflow_raw_protocol_header.header_protocol == SFLOW_HEADER_PROTOCOL_ETHERNET) {
 
-                bool unpack_gre = false;
-
-                // We could enable this new parser for testing purpose
-                auto result = parse_raw_packet_to_simple_packet_full_ng(header_payload_pointer,
-                                                                        sflow_raw_protocol_header.frame_length_before_sampling,
-                                                                        sflow_raw_protocol_header.header_size, packet,
-                                                                        unpack_gre, sflow_read_packet_length_from_ip_header);
+                auto result =
+                    parse_raw_packet_to_simple_packet_full_ng(header_payload_pointer, sflow_raw_protocol_header.frame_length_before_sampling,
+                                                              sflow_raw_protocol_header.header_size, packet,
+                                                              fastnetmon_global_configuration.sflow_extract_tunnel_traffic,
+                                                              fastnetmon_global_configuration.sflow_read_packet_length_from_ip_header);
 
                 if (result != network_data_stuctures::parser_code_t::success) {
                     sflow_parse_error_nested_header++;
@@ -409,10 +410,11 @@ bool process_sflow_flow_sample(uint8_t* data_pointer,
                 sflow_ipv4_header_protocol++;
 
                 // We parse this packet using special version of our parser which looks only on IPv4 packet
-                auto result = parse_raw_ipv4_packet_to_simple_packet_full_ng(header_payload_pointer,
-                                                                             sflow_raw_protocol_header.frame_length_before_sampling,
-                                                                             sflow_raw_protocol_header.header_size, packet,
-                                                                             sflow_read_packet_length_from_ip_header);
+                auto result =
+                    parse_raw_ipv4_packet_to_simple_packet_full_ng(header_payload_pointer,
+                                                                   sflow_raw_protocol_header.frame_length_before_sampling,
+                                                                   sflow_raw_protocol_header.header_size, packet,
+                                                                   fastnetmon_global_configuration.sflow_read_packet_length_from_ip_header);
 
                 if (result != network_data_stuctures::parser_code_t::success) {
                     sflow_parse_error_nested_header++;
@@ -435,16 +437,25 @@ bool process_sflow_flow_sample(uint8_t* data_pointer,
                 return false;
             }
 
+            // That's actually WEIRD as we touch these fields in packet parser logic too.
+            // So basically we do not use logic from parsers above and override values after parser finishes work
+	    
             // Pass pointer to raw header to FastNetMon processing functions
-            packet.payload_pointer     = header_payload_pointer;
-            packet.payload_full_length = sflow_raw_protocol_header.frame_length_before_sampling;
-            packet.captured_payload_length      = sflow_raw_protocol_header.header_size;
+            packet.payload_pointer         = header_payload_pointer;
+            packet.payload_full_length     = sflow_raw_protocol_header.frame_length_before_sampling;
+            packet.captured_payload_length = sflow_raw_protocol_header.header_size;
 
             packet.sample_ratio = sflow_sample_header_unified_accessor.sampling_rate;
 
             packet.input_interface  = sflow_sample_header_unified_accessor.input_port_index;
             packet.output_interface = sflow_sample_header_unified_accessor.output_port_index;
 
+            // In sFlow we have special way to learn that packet was discarded from type of output interface
+            if (sflow_sample_header_unified_accessor.output_port_type == SFLOW_PORT_FORMAT_PACKET_DISCARDED) {
+                packet.forwarding_status = forwarding_status_t::dropped;
+                sflow_packets_discarded++;
+            }
+
             // std::cout << print_simple_packet(packet) << std::endl;
         } else if (record_type == SFLOW_RECORD_TYPE_EXTENDED_ROUTER_DATA) {
             sflow_extended_router_data_records++;
@@ -472,7 +483,7 @@ bool process_sflow_flow_sample(uint8_t* data_pointer,
                 }
 
                 // We're ready to parse it
-                sflow_extended_gateway_information_t* gateway_details = (sflow_extended_gateway_information_t*)payload_ptr;
+                const sflow_extended_gateway_information_t* gateway_details = (const sflow_extended_gateway_information_t*)payload_ptr;
 
                 packet.src_asn = fast_ntoh(gateway_details->router_asn);
                 packet.dst_asn = fast_ntoh(gateway_details->source_asn);
@@ -493,9 +504,9 @@ bool process_sflow_flow_sample(uint8_t* data_pointer,
 // Awesome description about v5 format from AMX-IX folks:
 // Header structure from AMS-IX folks:
 // http://www.sflow.org/developers/diagrams/sFlowV5Datagram.pdf
-void parse_sflow_v5_packet(uint8_t* payload_ptr, unsigned int payload_length, uint32_t client_ipv4_address) {
+void parse_sflow_v5_packet(const uint8_t* payload_ptr, unsigned int payload_length, uint32_t client_ipv4_address) {
     sflow_packet_header_unified_accessor sflow_header_accessor;
-    uint8_t* total_packet_end = payload_ptr + payload_length;
+    const uint8_t* total_packet_end = payload_ptr + payload_length;
 
     // Increase total number of packets
     sflow_total_packets++;
@@ -503,14 +514,24 @@ void parse_sflow_v5_packet(uint8_t* payload_ptr, unsigned int payload_length, ui
     bool read_sflow_header_result = read_sflow_header(payload_ptr, payload_length, sflow_header_accessor);
 
     if (!read_sflow_header_result) {
-        logger << log4cpp::Priority::ERROR << plugin_log_prefix << "could not read sflow packet header correctly";
+        logger << log4cpp::Priority::ERROR << plugin_log_prefix << "could not read sFlow packet header correctly";
         sflow_bad_packets++;
         return;
     }
 
     if (sflow_header_accessor.get_datagram_samples_count() <= 0) {
         logger << log4cpp::Priority::ERROR << plugin_log_prefix
-               << "Strange number of sFLOW samples: " << sflow_header_accessor.get_datagram_samples_count();
+               << "Strange number of sFlow samples: " << sflow_header_accessor.get_datagram_samples_count();
+        sflow_bad_packets++;
+        return;
+    }
+
+    // As we're going to allocate memory using this value as number of elements 
+    // we need to ensure that we capped it by reasonable value
+    if (sflow_header_accessor.get_datagram_samples_count() > max_sflow_sample_number) {
+         logger << log4cpp::Priority::ERROR << plugin_log_prefix
+               << "Number of sFlow samples in packet " << sflow_header_accessor.get_datagram_samples_count()
+              << " exceeds allowed maximum value " << max_sflow_sample_number;
         sflow_bad_packets++;
         return;
     }
@@ -518,7 +539,7 @@ void parse_sflow_v5_packet(uint8_t* payload_ptr, unsigned int payload_length, ui
     std::vector<sample_tuple_t> samples_vector;
     samples_vector.reserve(sflow_header_accessor.get_datagram_samples_count());
 
-    uint8_t* samples_block_start = payload_ptr + sflow_header_accessor.get_original_payload_length();
+    const uint8_t* samples_block_start = payload_ptr + sflow_header_accessor.get_original_payload_length();
 
     bool discovered_padding = false;
 
@@ -541,49 +562,46 @@ void parse_sflow_v5_packet(uint8_t* payload_ptr, unsigned int payload_length, ui
         //    << " enterprise " << enterprise
         //    << " and length " << sample_length << std::endl;
 
-        int32_t enterprise     = std::get<0>(sample);
-        int32_t integer_format = std::get<1>(sample);
-        uint8_t* data_pointer  = std::get<2>(sample);
-        size_t data_length     = std::get<3>(sample);
+        int32_t enterprise          = std::get<0>(sample);
+        int32_t integer_format      = std::get<1>(sample);
+        const uint8_t* data_pointer = std::get<2>(sample);
+        size_t data_length          = std::get<3>(sample);
 
-        if (enterprise == 0) {
-            sflow_sample_type_t sample_format = sflow_sample_type_from_integer(integer_format);
+        if (enterprise != 0) {
+            // We do not support vendor specific additions
+            continue;
+        }
 
-            if (sample_format == sflow_sample_type_t::BROKEN_TYPE) {
-                logger << log4cpp::Priority::ERROR << plugin_log_prefix << "we got broken format type number: " << integer_format;
+        sflow_sample_type_t sample_format = sflow_sample_type_from_integer(integer_format);
 
-                return;
-            }
+        if (sample_format == sflow_sample_type_t::BROKEN_TYPE) {
+            logger << log4cpp::Priority::ERROR << plugin_log_prefix << "we got broken format type number: " << integer_format;
+            continue;
+        }
 
-            // Move this code to separate function!!!
-            if (sample_format == sflow_sample_type_t::FLOW_SAMPLE) {
-                // std::cout << "We got flow sample" << std::endl;
-                process_sflow_flow_sample(data_pointer, data_length, false, sflow_header_accessor, client_ipv4_address);
-                sflow_flow_samples++;
-            } else if (sample_format == sflow_sample_type_t::COUNTER_SAMPLE) {
-                // std::cout << "We got counter sample" << std::endl;
-                // TODO: add support for sflow counetrs
-                // process_sflow_counter_sample(data_pointer, data_length, false, sflow_header_accessor);
-                sflow_counter_sample++;
-            } else if (sample_format == sflow_sample_type_t::EXPANDED_FLOW_SAMPLE) {
-                // std::cout << "We got expanded flow sample" << std::endl;
-                process_sflow_flow_sample(data_pointer, data_length, true, sflow_header_accessor, client_ipv4_address);
-                sflow_flow_samples++;
-            } else if (sample_format == sflow_sample_type_t::EXPANDED_COUNTER_SAMPLE) {
-                // TODO:add support for sflow counetrs
-                // std::cout << "We got expanded counter sample" << std::endl;
-                ////process_sflow_counter_sample(data_pointer, data_length, true, sflow_header_accessor);
-                sflow_counter_sample++;
-            } else {
-                logger << log4cpp::Priority::ERROR << plugin_log_prefix << "we got broken format type: " << integer_format;
-            }
+        if (sample_format == sflow_sample_type_t::FLOW_SAMPLE) {
+            // logger << log4cpp::Priority::DEBUG << plugin_log_prefix << "We got flow sample";
+            process_sflow_flow_sample(data_pointer, data_length, false, sflow_header_accessor, client_ipv4_address);
+            sflow_flow_samples++;
+        } else if (sample_format == sflow_sample_type_t::COUNTER_SAMPLE) {
+            // logger << log4cpp::Priority::DEBUG << plugin_log_prefix << "We got counter sample";
+            process_sflow_counter_sample(data_pointer, data_length, false, sflow_header_accessor);
+            sflow_counter_sample++;
+        } else if (sample_format == sflow_sample_type_t::EXPANDED_FLOW_SAMPLE) {
+            // logger << log4cpp::Priority::DEBUG << plugin_log_prefix << "We got expanded flow sample";
+            process_sflow_flow_sample(data_pointer, data_length, true, sflow_header_accessor, client_ipv4_address);
+            sflow_flow_samples++;
+        } else if (sample_format == sflow_sample_type_t::EXPANDED_COUNTER_SAMPLE) {
+            // logger << log4cpp::Priority::DEBUG << plugin_log_prefix << "We got expanded counter sample";
+            // process_sflow_counter_sample(data_pointer, data_length, true, sflow_header_accessor);
+            sflow_expanded_counter_sample++;
         } else {
-            // do nothing because we haven't support for custom sFLOW data formats
+            logger << log4cpp::Priority::ERROR << plugin_log_prefix << "we got broken format type: " << integer_format;
         }
     }
 }
 
-bool process_sflow_counter_sample(uint8_t* data_pointer,
+bool process_sflow_counter_sample(const uint8_t* data_pointer,
                                   size_t data_length,
                                   bool expanded,
                                   const sflow_packet_header_unified_accessor& sflow_header_accessor) {
@@ -593,7 +611,7 @@ bool process_sflow_counter_sample(uint8_t* data_pointer,
         read_sflow_counter_header(data_pointer, data_length, expanded, sflow_counter_header_unified_accessor);
 
     if (!read_sflow_counter_header_result) {
-        logger << log4cpp::Priority::ERROR << plugin_log_prefix << "could not read sflow counter header";
+        logger << log4cpp::Priority::ERROR << plugin_log_prefix << "could not read sFlow counter header";
         return false;
     }
 
@@ -602,6 +620,14 @@ bool process_sflow_counter_sample(uint8_t* data_pointer,
         return false;
     }
 
+    if (sflow_counter_header_unified_accessor.get_number_of_counter_records() > max_number_of_counter_records) {
+        logger << log4cpp::Priority::ERROR << plugin_log_prefix << "number of counter records "
+                << sflow_counter_header_unified_accessor.get_number_of_counter_records()
+                << " exceeds maximum value "
+                << max_number_of_counter_records;
+        return false;
+    }
+
     std::vector<counter_record_sample_t> counter_record_sample_vector;
     counter_record_sample_vector.reserve(sflow_counter_header_unified_accessor.get_number_of_counter_records());
 
@@ -616,53 +642,45 @@ bool process_sflow_counter_sample(uint8_t* data_pointer,
         return false;
     }
 
-    for (auto counter_record : counter_record_sample_vector) {
-        uint32_t enterprise   = 0;
-        uint32_t format       = 0;
-        ssize_t length        = 0;
-        uint8_t* data_pointer = nullptr;
-
-        std::tie(enterprise, format, length, data_pointer) = counter_record;
-
-        if (enterprise == 0) {
-            sample_counter_types_t sample_type = sample_counter_types_t::BROKEN_COUNTER;
-            ;
-
-            if (format == 1) {
-                sample_type = sample_counter_types_t::GENERIC_INTERFACE_COUNTERS;
-            } else if (format == 2) {
-                sample_type = sample_counter_types_t::ETHERNET_INTERFACE_COUNTERS;
-            }
-
-            if (sample_type == sample_counter_types_t::ETHERNET_INTERFACE_COUNTERS) {
-                // std::cout << "ETHERNET_INTERFACE_COUNTERS" << std::endl;
-
-                if (sizeof(ethernet_sflow_interface_counters_t) != length) {
-                    logger << log4cpp::Priority::ERROR << plugin_log_prefix << "we haven't enough data for ethernet counter packet";
-                    return false;
-                }
-
-                ethernet_sflow_interface_counters_t ethernet_counters(data_pointer);
-                // std::cout << ethernet_counters.print() << std::endl;
-            }
-
-            if (sample_type == sample_counter_types_t::GENERIC_INTERFACE_COUNTERS) {
-                // std::cout << "GENERIC_INTERFACE_COUNTERS" << std::endl;
-
-                if (sizeof(generic_sflow_interface_counters_t) != length) {
-                    logger << log4cpp::Priority::ERROR << plugin_log_prefix << "we haven't enough data for generic packet";
-                    return false;
-                }
-
-                generic_sflow_interface_counters_t generic_sflow_interface_counters(data_pointer);
-                // std::cout << generic_sflow_interface_counters.print() << std::endl;
-            }
-        } else {
+    for (const auto& counter_record : counter_record_sample_vector) {
+        if (counter_record.enterprise != 0) {
             logger << log4cpp::Priority::ERROR << plugin_log_prefix << "we do not support vendor specific enterprise numbers";
+            continue;
         }
 
-        // std::cout << "Counter record" << std::endl;
+        sample_counter_types_t sample_type = sample_counter_types_t::BROKEN_COUNTER;
+
+        if (counter_record.format == 1) {
+            sample_type = sample_counter_types_t::GENERIC_INTERFACE_COUNTERS;
+        } else if (counter_record.format == 2) {
+            sample_type = sample_counter_types_t::ETHERNET_INTERFACE_COUNTERS;
+        }
+
+        if (sample_type == sample_counter_types_t::GENERIC_INTERFACE_COUNTERS) {
+            // logger << log4cpp::Priority::DEBUG << plugin_log_prefix << "GENERIC_INTERFACE_COUNTERS";
+
+            if (sizeof(generic_sflow_interface_counters_t) != counter_record.length) {
+                logger << log4cpp::Priority::ERROR << plugin_log_prefix
+                       << "length mismatch for generic interface counter packet: " << counter_record.length;
+                continue;
+            }
+
+            sflow_generic_interface_counter_sample++;
+
+            const generic_sflow_interface_counters_t* generic_sflow_interface_counters =
+                (const generic_sflow_interface_counters_t*)(counter_record.pointer);
+
+            if (logger.getPriority() == log4cpp::Priority::DEBUG) {
+                logger << log4cpp::Priority::DEBUG << generic_sflow_interface_counters->print();
+            }
+
+            // TODO: we need to get relevant fields in special intermediate structure and then pass them for processing
+            // somwhere else I think we need one dedicated thread to implement such metrics pusshes to Clickhouse
+        } else if (sample_type == sample_counter_types_t::ETHERNET_INTERFACE_COUNTERS) {
+            // These counters provide too detailed information about interface and we do not need it on such level for DDoS detection purposes
+        }
     }
 
     return true;
 }
+
